<?php

namespace Wb4Wp;

use Wb4Wp\Helpers\EnvironmentHelper;
use Wb4Wp\Managers\AdminManager;
use Wb4Wp\Managers\ApiManager;
use Wb4Wp\Managers\ProviderBluehostManager;
use Wb4Wp\Managers\ProviderWb4WpManager;
use Wb4Wp\Managers\DescriptionManager;
use Wb4Wp\Managers\StatsManager;
use Wb4Wp\Managers\ProvisionManager;
use Wb4Wp\Managers\TemplateManager;
use Wb4Wp\Managers\UpdateManager;
use Wb4Wp\Managers\WordPressManager;
use Wb4Wp\Managers\ThemeManager;
use Wb4Wp\Managers\InstanceManager;

/**
 * Class Wb4Wp
 *
 * @package Wb4Wp
 */
final class Wb4Wp
{
    private static $instance;

    private $admin_manager,
            $api_manager,
            $description_manager,
            $template_manager,
            $bluehost_manager,
            $wb4wp_manager,
            $word_press_manager,
            $theme_manager,
            $environment_helper,
            $instance_manager,
            $stats_manager,
            $provision_manager,
            $update_manager;

    public function __construct()
    {
        if (current_user_can('administrator')) {
            $this->admin_manager = new AdminManager();
            $this->admin_manager->add_hooks();
        }

        $this->api_manager = new ApiManager();
        $this->description_manager = new DescriptionManager();
        $this->template_manager = new TemplateManager();
        $this->bluehost_manager = new ProviderBluehostManager();
        $this->wb4wp_manager = new ProviderWb4WpManager();
        $this->word_press_manager = new WordPressManager();
        $this->theme_manager = new ThemeManager();
        $this->environment_helper = new EnvironmentHelper();
        $this->instance_manager = new InstanceManager();
        $this->provision_manager = new ProvisionManager();
        $this->stats_manager = new StatsManager();
        $this->update_manager = new UpdateManager();
    }

    public function add_hooks()
    {
        if ($this->admin_manager !== null) {
            $this->admin_manager->add_hooks();
        }

        $this->api_manager->add_hooks();
        $this->template_manager->add_hooks();
        $this->description_manager->add_hooks();
        $this->update_manager->add_hooks();
        $this->theme_manager->add_hooks();
    }

    public function get_provision_manager()
    {
        return $this->provision_manager;
    }

    public function get_stats_manager()
    {
        return $this->stats_manager;
    }

    public function get_instance_manager()
    {
        return $this->instance_manager;
    }

    public function get_api_manager()
    {
        return $this->api_manager;
    }

    public function get_wb4wp_manager()
    {
        return $this->wb4wp_manager;
    }

    public function get_admin_manager()
    {
        return $this->admin_manager;
    }

    public function get_description_manager()
    {
        return $this->description_manager;
    }

    public function get_template_manager()
    {
        return $this->template_manager;
    }

    public function get_bluehost_manager()
    {
        return $this->bluehost_manager;
    }

    public function get_word_press_manager()
    {
        return $this->word_press_manager;
    }

    public function get_update_manager()
    {
        return $this->update_manager;
    }

    public static function get_instance()
    {
        if (self::$instance === null) {
            self::$instance = new Wb4Wp();
        }

        return self::$instance;
    }
}
