<?php

namespace Wb4Wp\Helpers;

/**
 * Class ProviderHelper
 *
 * @package WB4WP\Helpers
 */
final class ProviderHelper
{
    const WB4WP = 'wb4wp';
    const BLUEHOST = 'bluehost';

    private static $providers = [
        self::BLUEHOST => 'Bluehost Website Builder',
        self::WB4WP => 'WebsiteBuilder',
    ];

    /**
     * Retrieves the plugin provider
     *
     * @return false|mixed|string|void
     */
    public static function get_provider()
    {
        $fallback_provider = self::WB4WP;
        
        $env_file_provider = env('WB4WP_PROVIDER');

        if($env_file_provider) {
            $fallback_provider = $env_file_provider;
        }

        return OptionsHelper::get(WB4WP_PROVIDER, $fallback_provider);
    }

    /**
     * Retrieves the plugin provider's name
     *
     * @return string
     */
    public static function get_provider_name()
    {
        return self::$providers[self::get_provider()];
    }

    public static function is_bluehost()
    {
        return self::BLUEHOST === self::get_provider();
    }

    public static function is_wb_4_wp()
    {
        return self::WB4WP === self::get_provider();
    }

    /**
     * Returns a list of providers
     *
     * @return string[]
     */
    public static function get_providers()
    {
        return array_values(self::$providers);
    }
}