<?php

namespace Wb4Wp\Helpers;

use stdClass;

/**
 * The SaveSiteHelper will manage the admin section of the plugin.
 *
 * @package Wb4Wp\Helpers
 */
final class SaveSiteHelper
{
    // SonarQube Code Smell
    const DESCRIPTION = 'description';
    const TITLE = 'title';
    const BACKGROUND = 'background';
    const COMPANY_NAME = 'companyName';
    const SUBTITLE = 'subtitle';

    /**
     * Gets a string representing the global section.
     *
     * @return string|array
     */
    public static function get_global_sections()
    {
        /**
         * Global Sections schema for Swagger
         *
         * @OA\Schema (
         *  schema="GlobalSections",
         *  type="object",
         *  description="The global sections",
         *  @OA\Property (
         *   property="footer",
         *   type="object",
         *   @OA\Property (
         *    property="id",
         *    type="integer",
         *   ),
         *   @OA\Property (
         *    property="layout",
         *    type="object",
         *    @OA\Property (
         *     property="section",
         *     type="object",
         *     @OA\Property (
         *      property="id",
         *      type="string",
         *     ),
         *    ),
         *   ),
         *   @OA\Property (
         *    property="category",
         *    type="string",
         *   ),
         *   @OA\Property (
         *    property="binding",
         *    type="object",
         *    @OA\Property (
         *     property="title-navigation",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="title-social",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="description",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="title",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="titleNavigation",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="titleSocial",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="titleAddress",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="titleDescription",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="copyright",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="pagesMenu",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="_toggle",
         *     type="object",
         *     @OA\Property (
         *      property="description",
         *      type="boolean",
         *     ),
         *     @OA\Property (
         *      property="copyrightHelper",
         *      type="boolean",
         *     ),
         *     @OA\Property (
         *      property="sitemapToggle",
         *      type="boolean",
         *     ),
         *     @OA\Property (
         *      property="global.email",
         *      type="boolean",
         *     ),
         *     @OA\Property (
         *      property="global.accounts",
         *      type="boolean",
         *     ),
         *     @OA\Property (
         *      property="global.phone",
         *      type="boolean",
         *     ),
         *     @OA\Property (
         *      property="logoToggle",
         *      type="boolean",
         *     ),
         *     @OA\Property (
         *      property="dividerToggle",
         *      type="boolean",
         *     ),
         *    ),
         *    @OA\Property (
         *     property="addressToggle",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="background",
         *     type="object",
         *     @OA\Property (
         *      property="colorIndex",
         *      type="integer",
         *     ),
         *     @OA\Property (
         *      property="patternIndex",
         *      type="integer",
         *     ),
         *     @OA\Property (
         *      property="opacity",
         *      type="integer",
         *     ),
         *     @OA\Property (
         *      property="none",
         *      type="boolean",
         *     ),
         *    ),
         *    @OA\Property (
         *     property="copyrightHelper",
         *     type="string",
         *    ),
         *   ),
         *  ),
         * )
         */
        return get_option(WB4WP_GLOBAL_SECTIONS, array(
            "footer" => array(
                'id' => 1,
                'layout' =>
                    array(
                        'section' =>
                            array(
                                'id' => 'bajigu77',
                            ),
                    ),
                'category' => 'footers',
                'binding' =>
                    array(
                        'title-navigation' => 'Pages',
                        'title-social' => 'Follow us',
                        self::DESCRIPTION => '',
                        self::TITLE => '',
                        'titleNavigation' => 'Pages',
                        'titleSocial' => 'Follow us',
                        'titleAddress' => 'Address',
                        'titleDescription' => 'About us',
                        'copyright' => true,
                        'pagesMenu' => false,
                        '_toggle' =>
                            array(
                                self::DESCRIPTION => true,
                                'copyrightHelper' => true,
                                'sitemapToggle' => true,
                                'global.email' => true,
                                'global.accounts' => true,
                                'global.phone' => true,
                                'logoToggle' => true,
                                'dividerToggle' => true,
                            ),
                        'addressToggle' => true,
                        self::BACKGROUND =>
                            array(
                                'colorIndex' => 1,
                                'patternIndex' => 7,
                                'opacity' => 0,
                                'none' => true,
                            ),
                        'copyrightHelper' => '',
                    ),
            )
        ));
    }

    /**
     * Gets a string representing the theme.
     *
     * @return string|array
     */
    public static function get_theme()
    {
        /**
         * Theme Object schema for Swagger
         *
         * @OA\Schema (
         *  schema="ThemeObject",
         *  type="object",
         *  description="The theme settings",
         *  @OA\Property (
         *   property="colors",
         *   type="object",
         *   @OA\Property (
         *    property="text",
         *    type="array",
         *    @OA\Items (
         *     type="integer",
         *    ),
         *   ),
         *   @OA\Property (
         *    property="background",
         *    type="array",
         *    @OA\Items (
         *     type="integer",
         *    ),
         *   ),
         *   @OA\Property (
         *    property="accent",
         *    type="array",
         *    @OA\Items (
         *     type="array",
         *     @OA\Items (
         *      type="integer",
         *     ),
         *    ),
         *   ),
         *   @OA\Property (
         *    property="style",
         *    type="string",
         *   ),
         *  ),
         *  @OA\Property (
         *   property="fonts",
         *   type="object",
         *   @OA\Property (
         *    property="body",
         *    type="object",
         *    @OA\Property (
         *     property="name",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="weight",
         *     type="string",
         *    ),
         *   ),
         *   @OA\Property (
         *    property="heading",
         *    type="object",
         *    @OA\Property (
         *     property="name",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="weight",
         *     type="string",
         *    ),
         *   ),
         *   @OA\Property (
         *    property="fontSize",
         *    type="integer",
         *   ),
         *  ),
         *  @OA\Property (
         *   property="animations",
         *   type="object",
         *   @OA\Property (
         *    property="enabled",
         *    type="boolean",
         *   ),
         *  ),
         *  @OA\Property (
         *   property="roundedCorners",
         *   type="boolean",
         *  ),
         * )
         */
        return get_option(WB4WP_THEME, array(
            'colors' =>
                array(
                    'text' =>
                        array(
                            0 => 246,
                            1 => 246,
                            2 => 246,
                        ),
                    self::BACKGROUND =>
                        array(
                            0 => 255,
                            1 => 255,
                            2 => 255,
                        ),
                    'accent' =>
                        array(
                            0 =>
                                array(
                                    0 => 250,
                                    1 => 250,
                                    2 => 250,
                                ),
                            1 =>
                                array(
                                    0 => 255,
                                    1 => 255,
                                    2 => 255,
                                ),
                        ),
                    'style' => 'minimal',
                ),
            'fonts' =>
                array(
                    'body' =>
                        array(
                            'name' => 'Lato',
                            'weight' => '400',
                        ),
                    'heading' =>
                        array(
                            'name' => 'Arvo',
                            'weight' => '400',
                        ),
                    'fontSize' => 1,
                ),
            'animations' =>
                array(
                    'enabled' => false,
                ),
            'roundedCorners' => false,
        ));
    }

    /**
     * Gets a string representing the metadata.
     *
     * @return string|array
     */
    public static function get_metadata()
    {
        /**
         * Metadata schema for Swagger
         *
         * @OA\Schema (
         *  schema="MetaData",
         *  type="object",
         *  description="The meta data object",
         *  @OA\Property (
         *   property="siteDescription",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="partnerId",
         *   type="integer",
         *  ),
         *  @OA\Property (
         *   property="siteKeywords",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="socialShareImage",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="googleAnalyticsKey",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="faviconUrl",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="siteName",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="siteDomain",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="siteHeaderHtml",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="siteFooterHtml",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="language",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="companyName",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="templateName",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="topicId",
         *   type="integer",
         *  ),
         *  @OA\Property (
         *   property="verticalId",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="addedPageTypes",
         *   type="array",
         *   @OA\Items (
         *    type="string",
         *   ),
         *  ),
         * )
         */
        return get_option(WB4WP_METADATA, array(
            'siteDescription' => '',
            'partnerId' => 200,
            'siteKeywords' => '',
            'socialShareImage' => 'https://storage.googleapis.com/production-constantcontact-v1-0-1/901/207901/4di52sUo/2470b3163d1647b4aa451bcfce496a44',
            'googleAnalyticsKey' => '',
            'faviconUrl' => '',
            'siteName' => '',
            'siteDomain' => '',
            'siteHeaderHtml' => '',
            'siteFooterHtml' => '',
            'language' => 'en',
            self::COMPANY_NAME => 'Peter',
            'templateName' => 'business',
            'topicId' => 321,
            'verticalId' => '86b90fbc-5657-4393-ad6d-72d3c63d5391',
            'addedPageTypes' => array(),
        ));
    }

    /**
     * Gets a string representing the global binding.
     *
     * @return string|array
     */
    public static function get_global_binding()
    {
        /**
         * Global Binding schema for Swagger
         *
         * @OA\Schema (
         *  schema="GlobalBinding",
         *  type="object",
         *  description="The global binding data",
         *  @OA\Property (
         *   property="accounts",
         *   type="object",
         *  ),
         *  @OA\Property (
         *   property="email",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="openingHours",
         *   type="object",
         *  ),
         *  @OA\Property (
         *   property="phone",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="companyName",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="logo",
         *   type="object",
         *   @OA\Property (
         *    property="value",
         *    type="string",
         *   ),
         *  ),
         *  @OA\Property (
         *   property="title",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="callToAction",
         *   type="object",
         *   @OA\Property (
         *    property="source",
         *    type="string",
         *   ),
         *   @OA\Property (
         *    property="title",
         *    type="string",
         *   ),
         *  ),
         *  @OA\Property (
         *   property="legal",
         *   type="object",
         *   @OA\Property (
         *    property="privacyPolicy",
         *    type="object",
         *    @OA\Property (
         *     property="showCookieBanner",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="bannerPosition",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="bannerText",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="agreeButtonText",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="privacyPolicyText",
         *     type="string",
         *    ),
         *   ),
         *   @OA\Property (
         *    property="termsOfService",
         *    type="object",
         *    @OA\Property (
         *     property="termsOfServiceText",
         *     type="string",
         *    ),
         *   ),
         *  ),
         *  @OA\Property (
         *   property="description",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="coverPhoto",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="about",
         *   type="string",
         *  ),
         * )
         */
        return get_option(WB4WP_GLOBAL_BINDING, array(
            "accounts" => [
                'twitter' => 'https://twitter.com/bluehost?utm_source=bluehost-website-builder&utm_medium=social-buttons&utm_campaign=default-value',
                'instagram' => 'https://www.instagram.com/bluehost/?utm_source=bluehost-website-builder&utm_medium=social-buttons&utm_campaign=default-value',
            ],
            "email" => get_bloginfo('admin_email'),
            "openingHours" => new stdClass,
            "phone" => "",
            self::COMPANY_NAME => "",
            "logo" => array(
                "value" => "",
            ),
            self::TITLE => wp_title(),
            "callToAction" => array(
                "source" => "global.phone",
                self::TITLE => "Call",
            ),
            "legal" => array(
                'privacyPolicy' =>
                    array(
                        'showCookieBanner' => true,
                        'bannerPosition' => 'bottom',
                        'bannerText' => 'This site uses cookies',
                        'agreeButtonText' => 'I\'m okay with that',
                        'privacyPolicyText' => '',
                    ),
                'termsOfService' =>
                    array(
                        'termsOfServiceText' => '',
                    ),
            ),
            self::DESCRIPTION => "",
            "coverPhoto" => "",
            "about" => "",
        ));
    }

    /**
     * Gets a string representing the header section.
     *
     * @return string|array
     */
    public static function get_header_section()
    {
        /**
         * Header Section object for Swagger
         *
         * @OA\Schema (
         *  schema="HeaderSection",
         *  type="object",
         *  description="The header section object",
         *  @OA\Property (
         *   property="id",
         *   type="integer",
         *  ),
         *  @OA\Property (
         *   property="layout",
         *   type="object",
         *   @OA\Property (
         *    property="section",
         *    type="object",
         *    @OA\Property (
         *     property="id",
         *     type="string",
         *    )
         *   )
         *  ),
         *  @OA\Property (
         *   property="category",
         *   type="string",
         *  ),
         *  @OA\Property (
         *   property="binding",
         *   type="object",
         *   @OA\Property (
         *    property="title",
         *    type="string",
         *   ),
         *   @OA\Property (
         *    property="subtitle",
         *    type="string",
         *   ),
         *   @OA\Property (
         *    property="description",
         *    type="string",
         *   ),
         *   @OA\Property (
         *    property="buttons",
         *    type="array",
         *    @OA\Items (
         *     type="object",
         *    ),
         *   ),
         *   @OA\Property (
         *    property="fullPage",
         *    type="boolean",
         *   ),
         *   @OA\Property (
         *    property="_placeholders",
         *    type="object",
         *    @OA\Property (
         *     property="subtitle",
         *     type="string",
         *    ),
         *    @OA\Property (
         *     property="description",
         *     type="string",
         *    ),
         *   ),
         *   @OA\Property (
         *    property="_toggle",
         *    type="object",
         *    @OA\Property (
         *     property="title",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="subtitle",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="description",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="buttons",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="global.logo",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="global.accounts",
         *     type="boolean",
         *    ),
         *    @OA\Property (
         *     property="global.title",
         *     type="boolean",
         *    ),
         *   ),
         *   @OA\Property (
         *    property="arrow",
         *    type="boolean",
         *   ),
         *   @OA\Property (
         *    property="fixedNavigation",
         *    type="boolean",
         *   ),
         *   @OA\Property (
         *    property="fitText",
         *    type="boolean",
         *   ),
         *   @OA\Property (
         *    property="contentAlignment",
         *    type="string",
         *   ),
         *   @OA\Property (
         *    property="background",
         *    type="object",
         *    @OA\Property (
         *     property="colorIndex",
         *     type="integer",
         *    ),
         *    @OA\Property (
         *     property="img",
         *     type="object",
         *     @OA\Property (
         *      property="value",
         *      type="string",
         *     ),
         *    ),
         *    @OA\Property (
         *     property="opacity",
         *     type="integer",
         *    ),
         *    @OA\Property (
         *     property="color",
         *     type="array",
         *     @OA\Items (
         *      type="object",
         *     ),
         *    ),
         *    @OA\Property (
         *     property="none",
         *     type="boolean",
         *    ),
         *   ),
         *   @OA\Property (
         *    property="cover",
         *    type="boolean",
         *   ),
         *   @OA\Property (
         *    property="companyName",
         *    type="string",
         *   ),
         *  )
         * )
         */
        return get_option(WB4WP_HEADER_SECTION, array(
            'id' => 22,
            'layout' =>
                array(
                    'section' =>
                        array(
                            'id' => 'bajigu78',
                        ),
                ),
            'category' => 'headers',
            'binding' =>
                array(
                    self::TITLE => 'Click here to edit your title',
                    self::SUBTITLE => 'Click here to edit your subtitle',
                    self::DESCRIPTION => 'Click here to edit your description',
                    'buttons' =>
                        array(),
                    'fullPage' => true,
                    '_placeholders' =>
                        array(
                            self::SUBTITLE => 'It\'s nice to meet you',
                            self::DESCRIPTION => 'Write something here to introduce yourself to your audience.',
                        ),
                    '_toggle' =>
                        array(
                            self::TITLE => true,
                            self::SUBTITLE => true,
                            self::DESCRIPTION => true,
                            'buttons' => true,
                            'global.logo' => false,
                            'global.accounts' => true,
                            'global.title' => true,
                        ),
                    'arrow' => true,
                    'fixedNavigation' => true,
                    'fitText' => true,
                    'contentAlignment' => 'align-center',
                    self::BACKGROUND =>
                        array(
                            'colorIndex' => 0,
                            'img' =>
                                array(
                                    'value' => 'https://images.builderservices.io/s/?https://images.unsplash.com/photo-1493836512294-502baa1986e2?ixlib=rb-1.2.1&q=80&fm=jpg&crop=entropy&cs=tinysrgb&w=1080&fit=max&ixid=eyJhcHBfaWQiOjU1MTN9',
                                ),
                            'opacity' => 50,
                            'color' =>
                                array(),
                            'none' => true,
                        ),
                    'cover' => true,
                    self::COMPANY_NAME => 'roberts pstcologist work',
                ),
        ));
    }

}
