<?php

namespace Wb4Wp\Helpers;

/**
 * Class SiteModelHelper
 *
 * @package Wb4Wp\Helpers
 */
final class SiteModelHelper
{
    public static function applyThemeModsToSiteModel( &$site_model, $get_setting )
    {
        $global_binding = &$site_model['globalBinding'];

        $main_page = null;
        foreach ( $site_model['pages'] as &$page ) {
            if ( !$page['mainPage'] ) {
                continue;
            }

            $main_page = &$page;
            break;
        }

        $header_section = &$main_page['headerSection'];
        $footer_section = &$site_model['globalSections']['footer'];
        $theme_colors = &$site_model['theme']['colors'];
        $theme_fonts = &$site_model['theme']['fonts'];

        $mappings = [
            // Header section
            'wb4wp_header_section_layout_setting' => [&$header_section['layout']['section'], 'id'],
            'wb4wp_header_section_social_buttons_setting' => [&$header_section['binding']['_toggle'], 'global.accounts'],
            'wb4wp_header_section_site_title_setting' => [&$header_section['binding']['_toggle'], 'global.title'],
            'wb4wp_header_section_fixed_navigation_bar_setting' => [&$header_section['binding'], 'fixedNavigation'],

            // Footer ection
            'wb4wp_footer_section_layout_setting' => [&$footer_section['layout']['section'], 'id'],
            'wb4wp_footer_section_social_buttons_setting' => [&$footer_section['binding']['_toggle'], 'global.accounts'],
            'wb4wp_footer_section_site_title_setting' => [&$footer_section['binding']['_toggle'], 'global.title'],
            'wb4wp_footer_section_description_setting' => [&$footer_section['binding'], 'description'],
            'wb4wp_footer_section_copyright_message_setting' => [&$footer_section['binding']['_toggle'], 'copyrightHelper'],
            'wb4wp_footer_section_link_to_sitemap_setting' => [&$footer_section['binding']['_toggle'], 'sitemapToggle'],
            'wb4wp_footer_section_page_menu_setting' => [&$footer_section['binding'], 'pagesMenu'],

            // Social accounts sections
            'wb4wp_social_accounts_section_facebook_setting' => [&$global_binding['accounts'], 'facebook'],
            'wb4wp_social_accounts_section_twitter_setting' => [&$global_binding['accounts'], 'twitter'],
            'wb4wp_social_accounts_section_instagram_setting' => [&$global_binding['accounts'], 'instagram'],
            'wb4wp_social_accounts_section_linkedin_setting' => [&$global_binding['accounts'], 'linkedin'],
            'wb4wp_social_accounts_section_pinterest_setting' => [&$global_binding['accounts'], 'pinterest'],
            'wb4wp_social_accounts_section_youtube_setting' => [&$global_binding['accounts'], 'youtube'],
            'wb4wp_social_accounts_section_opentable_setting' => [&$global_binding['accounts'], 'opentable'],

            // Contact information section
            'wb4wp_contact_information_section_phone_number_setting' => [&$global_binding, 'phone'],
            'wb4wp_contact_information_section_email_setting' => [&$global_binding, 'email'],
            'wb4wp_contact_information_section_street_setting' => [&$global_binding['address'], 'street'],
            'wb4wp_contact_information_section_city_setting' => [&$global_binding['address'], 'city'],
            'wb4wp_contact_information_section_zip_code_setting' => [&$global_binding['address'], 'zip'],
            'wb4wp_contact_information_section_state_setting' => [&$global_binding['address'], 'state'],
            'wb4wp_contact_information_section_country_setting' => [&$global_binding['address'], 'country'],

            // Color section
            'wb4wp_color_section_accent1_setting' => [&$theme_colors['accent'], 0],
            'wb4wp_color_section_accent2_setting' => [&$theme_colors['accent'], 1],
            'wb4wp_color_section_background_setting' => [&$theme_colors, 'background'],
            'wb4wp_color_section_text_setting' => [&$theme_colors, 'text'],

            // Fonts section
            'wb4wp_fonts_section_body_setting' => [&$theme_fonts, 'body'],
            'wb4wp_fonts_section_heading_setting' => [&$theme_fonts, 'heading'],
            'wb4wp_fonts_section_font_size_setting' => [&$theme_fonts, 'fontSize'],

            // Logo Section
            'wb4wp_logo_section_url_setting' => [&$global_binding['logo'], 'value'],
            'wb4wp_logo_section_show_in_header_setting' => [&$header_section['binding']['_toggle'], 'global.logo'],
            'wb4wp_logo_section_show_in_footer_setting' => [&$footer_section['binding']['_toggle'], 'global.logo'],
        ];

        foreach ( $mappings as $theme_mod_key => $property_path ) {
            $setting = $get_setting( $theme_mod_key );

            if ( !isset( $setting ) ) {
                continue;
            }

            if ( strpos( $theme_mod_key, 'social_accounts_section' ) !== false && empty( $setting ) ) {
                continue;
            }

            $parent = &$property_path[0];
            $key = $property_path[1];

            switch ( $theme_mod_key ) {
                case 'wb4wp_footer_section_layout_setting':
                    $setting = self::map_wp_footer_layout_id_to_ee( $setting );
                    break;

                case 'wb4wp_color_section_accent1_setting':
                case 'wb4wp_color_section_accent2_setting':
                case 'wb4wp_color_section_background_setting':
                case 'wb4wp_color_section_text_setting':
                    $setting = ColorHelper::hex_to_rgb( $setting );
                    break;

                case 'wb4wp_fonts_section_body_setting':
                case 'wb4wp_fonts_section_heading_setting':
                    $setting = self::map_wp_font_to_ee( $setting );
                    break;
            }

            if ( empty( $parent ) || !is_array( $parent ) ) {
                $parent = [$key => $setting];
            } else {
                $parent[$key] = $setting;
            }
        }

        $site_model['theme']['fonts']['customFont'] = true;
    }

    private static function map_wp_footer_layout_id_to_ee( $wp_footer_layout_id )
    {
        switch ( $wp_footer_layout_id ) {
            case 'footer-2':
                return 'mudoce19';
            case 'footer-3':
                return 'bajigu77';
            case 'footer-4':
                return 'bajigu78';
            case 'footer-5':
                return 'bajigu79';
            case 'footer-1':
            default:
                return 'bajigu80';
        }
    }

    private static function map_wp_font_to_ee( $wp_font )
    {
        $split = explode( ':', $wp_font );
        return [
            'name' => $split[0],
            'weight' => $split[1]
        ];
    }

    public static function mapSiteModelToThemeMods( $site_model )
    {
        $footer_section = $site_model['globalSections']['footer'];
        $global_binding = $site_model['globalBinding'];
        $main_pages = array_filter( $site_model['pages'], function ( $page ) {
            return $page['mainPage'];
        } );
        $main_page = current( $main_pages );
        $header_section = $main_page['headerSection'];
        $theme_colors = $site_model['theme']['colors'];
        $theme_fonts = $site_model['theme']['fonts'];

        $theme_mods = [];

        // Header section
        $theme_mods['wb4wp_header_section_layout_setting'] = $header_section['layout']['section']['id'];
        $theme_mods['wb4wp_header_section_social_buttons_setting'] = !empty( $header_section['binding']['_toggle']['global.accounts'] ) ? 'true' : 'false';
        $theme_mods['wb4wp_header_section_site_title_setting'] = !empty( $header_section['binding']['_toggle']['global.title'] ) ? 'true' : 'false';
        $theme_mods['wb4wp_header_section_fixed_navigation_bar_setting'] = !empty( $header_section['binding']['fixedNavigation'] ) ? 'true' : 'false';

        // Footer section
        $theme_mods['wb4wp_footer_section_layout_setting'] = self::map_ee_footer_layout_id_to_wp( $footer_section['layout']['section']['id'] );
        $theme_mods['wb4wp_footer_section_social_buttons_setting'] = !empty( $footer_section['binding']['_toggle']['global.accounts'] ) ? 'true' : 'false';
        $theme_mods['wb4wp_footer_section_site_title_setting'] = !empty( $footer_section['binding']['_toggle']['global.title'] ) ? 'true' : 'false';
        $theme_mods['wb4wp_footer_section_description_setting'] = isset( $footer_section['binding']['description'] ) ? $footer_section['binding']['description'] : '';
        $theme_mods['wb4wp_footer_section_copyright_message_setting'] = !empty( $footer_section['binding']['_toggle']['copyrightHelper'] ) ? 'true' : 'false';
        $theme_mods['wb4wp_footer_section_link_to_sitemap_setting'] = !empty( $footer_section['binding']['_toggle']['sitemapToggle'] ) ? 'true' : 'false';
        $theme_mods['wb4wp_footer_section_page_menu_setting'] = !empty( $footer_section['binding']['pagesMenu'] ) ? 'true' : 'false';

        // Logo section
        $theme_mods['wb4wp_logo_section_url_setting'] = !empty( $global_binding['logo']['value'] ) ? $global_binding['logo']['value'] : '';
        $theme_mods['wb4wp_logo_section_show_in_header_setting'] = !empty( $header_section['binding']['_toggle']['global.logo'] ) ? 'true' : 'false';
        $theme_mods['wb4wp_logo_section_show_in_footer_setting'] = !empty( $footer_section['binding']['_toggle']['global.logo'] ) ? 'true' : 'false';

        // Social accounts section
        $social_accounts = [
            'facebook',
            'twitter',
            'instagram',
            'linkedin',
            'pinterest',
            'youtube',
            'opentable',
        ];
        foreach ( $social_accounts as $social_account ) {
            if ( empty( $global_binding['accounts'][$social_account] ) ) {
                continue;
            }

            $theme_mods['wb4wp_social_accounts_section_' . $social_account . '_setting'] = $global_binding['accounts'][$social_account];
        }

        // Contact information section
        $theme_mods['wb4wp_contact_information_section_phone_number_setting'] = isset( $global_binding['phone'] ) ? $global_binding['phone'] : '';
        $theme_mods['wb4wp_contact_information_section_email_setting'] = isset( $global_binding['email'] ) ? $global_binding['email'] : '';
        $theme_mods['wb4wp_contact_information_section_street_setting'] = isset( $global_binding['address']['street'] ) ? $global_binding['address']['street'] : '';
        $theme_mods['wb4wp_contact_information_section_city_setting'] = isset( $global_binding['address']['city'] ) ? $global_binding['address']['city'] : '';
        $theme_mods['wb4wp_contact_information_section_zip_code_setting'] = isset( $global_binding['address']['zip'] ) ? $global_binding['address']['zip'] : '';
        $theme_mods['wb4wp_contact_information_section_state_setting'] = isset( $global_binding['address']['state'] ) ? $global_binding['address']['state'] : '';
        $theme_mods['wb4wp_contact_information_section_country_setting'] = isset( $global_binding['address']['country'] ) ? $global_binding['address']['country'] : '';

        // Colors section
        if ( isset( $theme_colors['accent'][0] ) ) {
            $theme_mods['wb4wp_color_section_accent1_setting'] = ColorHelper::rgb_to_hex( $theme_colors['accent'][0] );
        }

        if ( isset( $theme_colors['accent'][1] ) ) {
            $theme_mods['wb4wp_color_section_accent2_setting'] = ColorHelper::rgb_to_hex( $theme_colors['accent'][1] );
        }

        if ( isset( $theme_colors['background'] ) ) {
            $theme_mods['wb4wp_color_section_background_setting'] = ColorHelper::rgb_to_hex( $theme_colors['background'] );
        }

        if ( isset( $theme_colors['text'] ) ) {
            $theme_mods['wb4wp_color_section_text_setting'] = ColorHelper::rgb_to_hex( $theme_colors['text'] );
        }

        // Fonts section
        if ( isset( $theme_fonts['body'] ) ) {
            $theme_mods['wb4wp_fonts_section_body_setting'] = self::map_ee_font_to_wp( $theme_fonts['body'] );
        }

        if ( isset( $theme_fonts['heading'] ) ) {
            $theme_mods['wb4wp_fonts_section_heading_setting'] = self::map_ee_font_to_wp( $theme_fonts['heading'] );
        }

        if ( isset( $theme_fonts['fontSize'] ) ) {
            $theme_mods['wb4wp_fonts_section_font_size_setting'] = $theme_fonts['fontSize'];
        }

        return $theme_mods;
    }

    private static function map_ee_footer_layout_id_to_wp( $ee_footer_layout_id )
    {
        switch ( $ee_footer_layout_id ) {
            case 'mudoce19':
                return 'footer-2';
            case 'bajigu77':
                return 'footer-3';
            case 'bajigu78':
                return 'footer-4';
            case 'bajigu79':
                return 'footer-5';
            case 'bajigu80':
            default:
                return 'footer-1';
        }
    }

    private static function map_ee_font_to_wp( $ee_font )
    {
        return $ee_font['name'] . ':' . $ee_font['weight'];
    }

}