<?php

namespace Wb4Wp\Managers;

use DOMDocument;
use Exception;
use stdClass;
use Wb4Wp\Constants\ThemeConstants;
use Wb4Wp\Helpers\OptionsHelper;
use Wb4Wp\Helpers\ProviderHelper;
use Wb4Wp\Helpers\SaveSiteHelper;
use Wb4Wp\Helpers\SiteModelHelper;
use Wb4WpTheme\Managers\CustomizeManager;
use WP_REST_Response;

/**
 * The DescriptionManager will manage the description of the website.
 *
 * @ignore
 */
final class DescriptionManager
{
    const GLOBAL_BINDING = 'globalBinding';
    const METADATA = 'metadata';
    const LANGUAGE = 'language';

    // SonarQube dictionary
    const POST_STATUS = 'post_status';
    const POST_TYPE = 'post_type';
    const POST_DRAFT = 'draft';
    const POST_PUBLISH = 'publish';
    const WB4WP_NAV_ID = 'WB4WP_NAV_ID';
    const WB4WP_PAGE_ID = 'WB4WP_PAGE_ID';
    const WB4WP_PAGE_SECTIONS = 'WB4WP_PAGE_SECTIONS';
    const WB4WP_DRAFT_SECTIONS = 'WB4WP_DRAFT_SECTIONS';
    const WB4WP_PAGE_MATCH_URI_TO_TITLE = 'WB4WP_PAGE_MATCH_URI_TO_TITLE';
    const WB4WP_EDIT_MODE = 'WB4WP_EDIT_MODE';
    const TITLE = 'title';
    const URI_PATH = 'uriPath';
    const SECTION = 'section';
    const SUBTITLE = 'subtitle';
    const DESCRIPTION = 'description';
    const PAGE_HOME = 'page:home';
    const LINK_TYPE = 'linkType';
    const STYLES = 'styles';
    const BACKGROUND = 'background';
    const BORDER = 'border';
    const READ_MORE = 'Read more';
    const SECTIONS = 'sections';
    const MAIN_PAGE = 'mainPage';
    const CAN_BE_DELETED = 'can_be_deleted';
    const WP_PAGE_TEMPLATE = '_wp_page_template';
    const GLOBAL_SECTIONS = 'globalSections';
    const HEADER_SECTION = 'headerSection';
    const BINDING = 'binding';
    const THEME = 'theme';
    const PAGES = 'pages';
    const WP_FILTER_POST_KSES = 'wp_filter_post_kses';
    const FONTS = 'fonts';
    const JSON_EXT = '.json';
    const BODY = 'body';
    const NAME = 'name';
    const VALUE = 'value';
    const COLORS = 'colors';
    const IMAGE = 'image';
    const HEADING = 'heading';
    const APPEARANCE_FONT_TITLE = 'font_title';
    const APPEARANCE_FONT_BODY = 'font_body';
    const APPEARANCE_COLORS = 'colors';
    const APPEARANCE_FEATURED_IMAGE = 'featured_image';
    const FAVICON_URL = 'faviconUrl';
    const TOGGLE = '_toggle';
    const MENU_ITEM = 'menuItem';
    const SHOW_IN_NAVIGATION = 'showInNavigation';
    const SITE_MODEL = 'siteModel';
    const SITE_NAME = 'siteName';

    const ASSETS_PATH = '/page-assets/assets_';
    const DEFAULT_SECTIONS = [
        0 => [
            'id' => 189,
            'layout' => [
                self::SECTION => [
                    'id' => 'hagopy24',
                ],
            ],
            'category' => 'text',
            self::BINDING => [
                self::TITLE => 'Welcome to our website',
                self::SUBTITLE => 'Learn more about what we do',
                self::DESCRIPTION => 'You can edit text on your website by double clicking on a text box on your website. Alternatively, when you select a text box a settings menu will appear. Selecting \'Edit Text\' from this menu will also allow you to edit the text within this text box. Remember to keep your wording friendly, approachable and easy to understand as if you were talking to your customer. You can edit text on your website by double clicking on a text box on your website. Alternatively, when you select a text box a settings menu will appear. Selecting \'Edit Text\' from this menu will also allow you to edit the text within this text box. Remember to keep your wording friendly, approachable and easy to understand as if you were talking to your customer',
                'buttons' => [
                    0 => [
                        'href' => self::PAGE_HOME,
                        'id' => '01',
                        self::LINK_TYPE => 'page',
                        self::STYLES => [
                            self::BACKGROUND => '',
                            self::BORDER => '',
                        ],
                        self::BACKGROUND => '',
                        self::BORDER => '',
                        self::TITLE => self::READ_MORE,
                    ],
                ],
                'icon' => [
                    self::VALUE => '',
                ],
                'list' => [
                    0 => [
                        self::TITLE => 'Add a title',
                        self::DESCRIPTION => 'You can use this element to explain to visitors what you do or inform them about other subjects. For instance, what is your passion and why or what does your company offer, i.e. products and services. You can hide this element in the menu on the right',
                        'button' => [
                            'href' => self::PAGE_HOME,
                            self::LINK_TYPE => 'page',
                            self::STYLES => [
                                self::BACKGROUND => '',
                                self::BORDER => '',
                            ],
                            self::BACKGROUND => '',
                            self::BORDER => '',
                            self::TITLE => self::READ_MORE,
                        ],
                    ],
                    1 => [
                        self::TITLE => 'Add a title',
                        self::DESCRIPTION => 'You can use this element to explain to visitors what you do or inform them about other subjects. For instance, what is your passion and why or what does your company offer, i.e. products and services. You can hide this element in the menu on the right',
                        'button' => [
                            'href' => self::PAGE_HOME,
                            self::LINK_TYPE => 'page',
                            self::STYLES => [
                                self::BACKGROUND => '',
                                self::BORDER => '',
                            ],
                            self::BACKGROUND => '',
                            self::BORDER => '',
                            self::TITLE => self::READ_MORE,
                        ],
                    ],
                ],
                self::IMAGE => [
                    self::VALUE => 'https://images.unsplash.com/photo-1514672013381-c6d0df1c8b18?ixlib=rb-1.2.1&q=80&fm=jpg&crop=entropy&cs=tinysrgb&w=1080&fit=max&ixid=eyJhcHBfaWQiOjU1MTN9',
                    'altText' => 'men\'s black hair',
                    'originalSize' => [
                        'width' => 6000,
                        'height' => 4000,
                    ],
                ],
                '_placeholders' => [
                    self::SUBTITLE => 'Learn more about what we do',
                    self::DESCRIPTION => 'You can edit text on your website by double clicking on a text box on your website. Alternatively, when you select a text box a settings menu will appear. Selecting \'Edit Text\' from this menu will also allow you to edit the text within this text box. Remember to keep your wording friendly, approachable and easy to understand as if you were talking to your customer. You can edit text on your website by double clicking on a text box on your website. Alternatively, when you select a text box a settings menu will appear. Selecting \'Edit Text\' from this menu will also allow you to edit the text within this text box. Remember to keep your wording friendly, approachable and easy to understand as if you were talking to your customer',
                ],
                self::TOGGLE => [
                    self::TITLE => true,
                    self::SUBTITLE => true,
                    self::DESCRIPTION => true,
                ],
                'contentAlignment' => 'align-center',
                'sectionSize' => 'section--lg',
                self::BACKGROUND => [
                    'colorIndex' => 0,
                    'none' => true,
                ],
            ],
        ],
    ];

    public $id_counter;

    /**
     * PageManager constructor.
     */
    public function __construct()
    {
        $this->id_counter = (int)date_timestamp_get(date_create());
    }

    /**
     * Will register all the hooks for the pages portion of the plugin.
     */
    public function add_hooks()
    {
        add_action('before_delete_post', [$this, 'remove_page_assets']);
        add_action('wp_head', [$this, 'add_assets_to_head']);
        add_action('wp_footer', [$this, 'add_assets_to_footer']);

        add_filter('bloginfo', [$this, 'set_blog_info'], 10, 2);
        add_filter('locale', function ($locale) {
            $meta_data = OptionsHelper::get(WB4WP_METADATA, null);

            if (!empty($meta_data) && !empty($meta_data[self::LANGUAGE])) {
                return $meta_data[self::LANGUAGE];
            }

            return $locale;
        });
    }

    /**
     * Hooks into the bloginfo function
     *
     * @param $text
     * @param $show
     *
     * @return false|mixed|void
     */
    public function set_blog_info($text, $show)
    {
        if ('name' === $show) {
            $title = OptionsHelper::get(WB4WP_SITE_TITLE, '');
            if (!empty($title)) {
                return $title;
            }
        } else if (self::DESCRIPTION === $show) {
            $description = OptionsHelper::get(WB4WP_SITE_DESCRIPTION, '');
            if (!empty($description)) {
                return $description;
            }
        }

        return $text;
    }

    public function get_navigation()
    {
        $pages = get_pages([self::POST_STATUS => [self::POST_DRAFT, self::POST_PUBLISH]]);
        $ee_navigation = [];

        foreach ($pages as $page) {
            $id = $page->ID;
            $ee_nav_id = get_post_meta($id, self::WB4WP_NAV_ID, true);

            if ($ee_nav_id == '') {
                $ee_nav_id = $this->id_counter++;
                update_post_meta($id, self::WB4WP_NAV_ID, $ee_nav_id);
            }

            if ($this->is_built_with_wb4wp_($id)) {
                $ee_id = get_post_meta($id, self::WB4WP_PAGE_ID, true);
                $ee_nav_id = get_post_meta($id, self::WB4WP_NAV_ID, true);

                array_push($ee_navigation, [
                    "id" => (int)$ee_nav_id,
                    "pageId" => $ee_id,
                    "wpID" => $id,
                    "parentId" => "hidden"
                ]);
            } else {
                $name = $page->post_title;
                $url = get_page_link($id);

                /**
                 * EE_Navigation_Item schema for Swagger
                 *
                 * @OA\Schema (
                 *  schema="EE_Navigation_Item",
                 *  type="object",
                 *  description="The Express Editor navigation object",
                 *  @OA\Property (
                 *   property="id",
                 *   type="integer",
                 *  ),
                 *  @OA\Property (
                 *   property="title",
                 *   type="string",
                 *  ),
                 *  @OA\Property (
                 *   property="order",
                 *   type="integer",
                 *  ),
                 *  @OA\Property (
                 *   property="menuItem",
                 *   type="object",
                 *   @OA\Property (
                 *    property="title",
                 *    type="string",
                 *   ),
                 *   @OA\Property (
                 *    property="uriPath",
                 *    type="string",
                 *   ),
                 *   @OA\Property (
                 *    property="showInNavigation",
                 *    type="boolean",
                 *   ),
                 *   @OA\Property (
                 *    property="externalUrlNewTab",
                 *    type="boolean",
                 *   ),
                 *   @OA\Property (
                 *    property="mainlink",
                 *    type="boolean",
                 *   ),
                 *  ),
                 * )
                 */
                array_push($ee_navigation, [
                    "id" => (int)$ee_nav_id,
                    "wpID" => $id,
                    self::TITLE => $name,
                    self::MENU_ITEM => [
                        self::TITLE => $name,
                        self::URI_PATH => $url,
                        self::SHOW_IN_NAVIGATION => false,
                        "externalUrlNewTab" => false,
                        "mainlink" => false,
                    ],
                ]);
            }
        }

        $menus = wp_get_nav_menus();
        $menuIndex = array_search(ProviderHelper::get_provider_name(), array_column($menus, 'name'));
        if ($menuIndex !== false) {
            $menu = $menus[$menuIndex];
            $menu_items = wp_get_nav_menu_items($menu);

            // Add all external links to the result
            foreach ($menu_items as $k => &$menu_item) {
                if ($menu_item->object === 'custom') {
                    array_push($ee_navigation, [
                        self::TITLE => $menu_item->title,
                        'id' => $menu_item->ID,
                        self::MENU_ITEM => [
                            self::TITLE => $menu_item->title,
                            self::URI_PATH => $menu_item->url,
                            self::SHOW_IN_NAVIGATION => true,
                            "externalUrlNewTab" => true,
                            "mainlink" => false,
                        ],
                    ]);
                }
            }

            $order = 1;
            foreach ($menu_items as $k => &$menu_item) {
                if ($menu_item->object === 'custom') {
                    $ee_navigation_item_index = array_search($menu_item->ID, array_column($ee_navigation, 'id'));
                } else {
                    $ee_id = intval($menu_item->object_id);
                    $ee_navigation_item_index = array_search($ee_id, array_column($ee_navigation, 'wpID'));
                }

                if ($ee_navigation_item_index !== false || $menu_item->object === 'custom') {
                    // Check if menu item is a sub child
                    $parentId = intval($menu_item->menu_item_parent);

                    if ($parentId > 0) {
                        // Find the parent item in the menu items array
                        $parent_index = array_search($parentId, array_column($menu_items, 'ID'));
                        if ($parent_index !== false) {
                            $parent_item = $menu_items[$parent_index];
                            // Use the object id to find that element in the EE navigation object
                            $ee_parent_id = intval($parent_item->object_id);
                            $ee_parent_navigation_item_index = array_search($ee_parent_id, array_column($ee_navigation, 'wpID'));
                            // Set the parent id of the EE navigation element
                            if ($ee_parent_navigation_item_index !== false) {
                                $ee_navigation[$ee_navigation_item_index]['parentId'] = $ee_navigation[$ee_parent_navigation_item_index]['id'];
                            } else {
                                $ee_navigation[$ee_navigation_item_index]['parentId'] = $parentId;
                            }
                        }
                    } else {
                        unset($ee_navigation[$ee_navigation_item_index]['parentId']);
                    }

                    $ee_navigation[$ee_navigation_item_index]['order'] = $order++;

                    if (!empty($ee_navigation[$ee_navigation_item_index][self::MENU_ITEM])) {
                        $ee_navigation[$ee_navigation_item_index][self::MENU_ITEM][self::SHOW_IN_NAVIGATION] = true;
                    }
                }
            }
        } else {
            foreach ($ee_navigation as $k => $nav_item) {
                unset($ee_navigation[$k]['parentId']);
            }
        }

        /**
         * EE_Navigation schema for Swagger
         *
         * @OA\Schema (
         *  schema="EE_Navigation",
         *  type="array",
         *  description="The array of Express Editor navigation items",
         *  @OA\Items (
         *   ref="#/components/schemas/EE_Navigation_Item",
         *  ),
         * )
         */
        return $ee_navigation;
    }

    public function get_pages()
    {
        $pages = get_pages([self::POST_STATUS => [self::POST_DRAFT, self::POST_PUBLISH]]);
        $ee_pages = [];
        $had_main_page = false;

        foreach ($pages as $page) {
            $is_main_page = $page->ID === (int) get_option( 'page_on_front', -1 );
            $name = $page->post_title;
            $id = $page->ID;

            if (!$this->is_built_with_wb4wp_($id)) {
                continue;
            }

            if ($is_main_page) {
                $had_main_page = true;
            }

            $ee_id = get_post_meta($id, self::WB4WP_PAGE_ID, true);
            $sections = get_post_meta($id, self::WB4WP_PAGE_SECTIONS, true);
            $revision_sections = get_post_meta($id, self::WB4WP_DRAFT_SECTIONS, true);

            if (!empty($revision_sections)) {
                $sections = $revision_sections;
            }

            $matchUriToTitle = get_post_meta($id, self::WB4WP_PAGE_MATCH_URI_TO_TITLE, true);

            if ($ee_id == '') {
                $ee_id = $this->id_counter++;
                update_post_meta($id, self::WB4WP_PAGE_ID, $ee_id);
            }

            if ($sections == '') {
                $sections = self::DEFAULT_SECTIONS;
                update_post_meta($id, self::WB4WP_PAGE_SECTIONS, $sections);
            }

            if ($matchUriToTitle === '') {
                $matchUriToTitle = false;
                update_post_meta($id, self::WB4WP_PAGE_MATCH_URI_TO_TITLE, $matchUriToTitle);
            }

            /**
             * EE_Page schema for Swagger
             *
             * @OA\Schema (
             *  schema="EE_Page",
             *  type="object",
             *  description="Express Editor page object",
             *  @OA\Property (
             *   property="sections",
             *   type="array",
             *   @OA\Items (
             *    type="string",
             *   ),
             *  ),
             *  @OA\Property (
             *   property="mainPage",
             *   type="boolean",
             *  ),
             *  @OA\Property (
             *   property="name",
             *   type="string",
             *  ),
             *  @OA\Property (
             *   property="id",
             *   type="integer",
             *  ),
             *  @OA\Property (
             *   property="wp_id",
             *   type="integer",
             *  ),
             *  @OA\Property (
             *   property="uriPath",
             *   type="string",
             *  ),
             *  @OA\Property (
             *   property="title",
             *   type="string",
             *  ),
             *  @OA\Property (
             *   property="matchUriToTitle",
             *   type="boolean"
             *  ),
             *  @OA\Property (
             *   property="showInNavigation",
             *   type="boolean",
             *  ),
             *  @OA\Property (
             *   property="headerSection",
             *   type="object",
             *  ),
             * )
             */
            array_push(
                $ee_pages,
                [
                    self::SECTIONS => $sections,
                    self::MAIN_PAGE => $is_main_page,
                    "name" => $name,
                    "id" => $ee_id,
                    "wp_id" => $id,
                    self::URI_PATH => $page->post_name,
                    self::TITLE => $name,
                    "matchUriToTitle" => $matchUriToTitle,
                    self::SHOW_IN_NAVIGATION => true,
                    self::HEADER_SECTION => SaveSiteHelper::get_header_section(),
                ]
            );
        }

        if (!$had_main_page && !empty($ee_pages[0])) {
            $ee_pages[0][self::MAIN_PAGE] = true;
        }

        $main_page = array_column($ee_pages, self::MAIN_PAGE);
        array_multisort($main_page, SORT_DESC, $ee_pages);

        /**
         * EE_Pages schema for Swagger
         *
         * @OA\Schema (
         *  schema="EE_Pages",
         *  type="array",
         *  description="The Express Editor pages",
         *  @OA\Items (
         *   ref="#/components/schemas/EE_Page",
         *  ),
         * )
         */
        return $ee_pages;
    }

    public function set_pages($ee_pages, $navigation_items, $publish_changes = true)
    {
        $ee_pages_mapping = $this->get_pages_mapping();
        $nav_pages = [];

        // Post filtering
        remove_filter('content_save_pre', self::WP_FILTER_POST_KSES);
        remove_filter('content_filtered_save_pre', self::WP_FILTER_POST_KSES);
        foreach ($ee_pages as $ee_page) {
            $ee_id = $ee_page['id'];
            $sections = $ee_page[self::SECTIONS];
            $is_main_page = !empty($ee_page[self::MAIN_PAGE]);
            $match_uri_to_title = $ee_page['matchUriToTitle'];
            $name = $ee_page[self::TITLE];
            $uri_path = $ee_page[self::URI_PATH];

            if ($is_main_page) {
                $header_section = $ee_page[self::HEADER_SECTION];
                OptionsHelper::set(WB4WP_HEADER_SECTION, $header_section);
            }

            if (array_key_exists($ee_id, $ee_pages_mapping)) {
                $ee_pages_mapping[$ee_id][self::CAN_BE_DELETED] = false;
                $id = $ee_pages_mapping[$ee_id]['id'];
                $page = get_post($id);
                array_push($nav_pages, [
                    "pageId" => $ee_id,
                    "wpId" => $id,
                ]);

                $page->post_title = $name;
                $page->post_name = $uri_path;

                if (!$publish_changes && $page->post_status === self::POST_PUBLISH) {
                    if (get_post_meta($page->ID, self::WB4WP_PAGE_SECTIONS, true) !== $sections) {
                        update_post_meta($page->ID, self::WB4WP_DRAFT_SECTIONS, $sections);
                    }

                    $page_arr = (array) $page;
                    $page_arr['post_name'] = $uri_path;
                    $page_arr['post_title'] = $name;

                    wp_update_post($page_arr);
                } else {
                    $page_arr = (array) $page;
                    $page_arr['meta_input'] = array(
                        self::WB4WP_PAGE_ID => $ee_id,
                        self::WB4WP_PAGE_SECTIONS => $sections,
                        self::WB4WP_PAGE_MATCH_URI_TO_TITLE => $match_uri_to_title,
                    );

                    wp_update_post($page_arr);
                }
            } else {
                $meta_values = array(
                    self::WB4WP_EDIT_MODE => 'builder',
                    self::WP_PAGE_TEMPLATE => 'wb4wp-template.php',
                    self::WB4WP_PAGE_ID => $ee_id,
                    self::WB4WP_PAGE_SECTIONS => $sections,
                    self::WB4WP_PAGE_MATCH_URI_TO_TITLE => $match_uri_to_title,
                );

                $id = wp_insert_post([
                    'post_title' => $name,
                    'post_content' => 'New Page',
                    'post_name' => $uri_path,
                    self::POST_STATUS => self::POST_DRAFT,
                    self::POST_TYPE => 'page',
                    'meta_input' => $meta_values,
                ]);

                array_push($nav_pages, [
                    "pageId" => $ee_id,
                    "wpId" => $id,
                ]);
            }
        }

        add_filter('content_save_pre', self::WP_FILTER_POST_KSES);
        add_filter('content_filtered_save_pre', self::WP_FILTER_POST_KSES);

        $navigation_items_filtered = array_values(array_filter( $navigation_items, function ( $navigation_item ) {
            return isset( $navigation_item['wpID'] );
        } ));
        foreach ($ee_pages_mapping as $ee_page_mapping) {
            if ($ee_page_mapping[self::CAN_BE_DELETED]) {
                wp_delete_post($ee_page_mapping['id']);
            } else {
                // Update changed page titles
                $nav_index = array_search($ee_page_mapping['id'], array_column($navigation_items_filtered, 'wpID'));
                if ($nav_index !== false) {
                    $nav_item = $navigation_items_filtered[$nav_index];

                    if (!empty($nav_item[self::MENU_ITEM])) {
                        wp_update_post([
                            'ID' => $ee_page_mapping['id'],
                            'post_title' => $nav_item[self::MENU_ITEM]['title'],
                        ]);
                    }
                }
            }
        }

        return $nav_pages;
    }

    public function handle_description_get()
    {
        try {
            $globalBinding = SaveSiteHelper::get_global_binding();

            $pages = $this->get_pages();

            // return a empty response if we dont have pages
            if (empty($pages)) {
                return ["success" => false];
            }

            $metadata = SaveSiteHelper::get_metadata();

            /**
             * Site Description schema for Swagger
             *
             * @OA\Schema (
             *  schema="SiteDescription",
             *  type="object",
             *  description="The site description model",
             *  @OA\Property (
             *   property="globalSections",
             *   ref="#/components/schemas/GlobalSections",
             *  ),
             *  @OA\Property (
             *   property="theme",
             *   ref="#/components/schemas/ThemeObject",
             *  ),
             *  @OA\Property (
             *   property="globalBinding",
             *   ref="#/components/schemas/GlobalBinding",
             *  ),
             *  @OA\Property (
             *   property="metadata",
             *   ref="#/components/schemas/MetaData",
             *  ),
             *  @OA\Property (
             *   property="pages",
             *   ref="#/components/schemas/EE_Pages",
             *  ),
             *  @OA\Property (
             *   property="navigation",
             *   ref="#/components/schemas/EE_Navigation",
             *  ),
             *  @OA\Property (
             *   property="apiKeys",
             *   type="object",
             *  ),
             *  @OA\Property (
             *   property="urls",
             *   type="object",
             *  ),
             *  @OA\Property (
             *   property="featureStorage",
             *   type="object",
             *  ),
             *  @OA\Property (
             *   property="globalFeatureModel",
             *   type="object",
             *  ),
             *  @OA\Property (
             *   property="language",
             *   type="string",
             *  ),
             * )
             */
            $description = [
                self::GLOBAL_SECTIONS => SaveSiteHelper::get_global_sections(),
                self::THEME => SaveSiteHelper::get_theme(),
                self::GLOBAL_BINDING => $globalBinding,
                self::METADATA => $metadata,
                'pages' => $pages,
                'navigation' => $this->get_navigation(),
                'apiKeys' => OptionsHelper::get(WB4WP_API_KEYS, new stdClass),
                'urls' => OptionsHelper::get(WB4WP_URLS, new stdClass),
                'featureStorage' => OptionsHelper::get(WB4WP_FEATURE_STORAGE, new stdClass),
                'globalFeatureModel' => OptionsHelper::get(WB4WP_GLOBAL_FEATURE_MODEL, new stdClass),
                self::LANGUAGE => 'en-US',
            ];

            if (class_exists('\Wb4WpTheme\Managers\CustomizeManager')) {
                SiteModelHelper::applyThemeModsToSiteModel(
                    $description,
                    function ( $setting_name ) {
                        try {
                            return CustomizeManager::get_setting( $setting_name );
                        } catch (Exception $exception) {
                            return null;
                        }
                    }
                );
            }

            /**
             * Site Model schema for Swagger
             *
             * @OA\Schema (
             *  schema="SiteModel",
             *  type="object",
             *  description="The site model",
             *  @OA\Property (
             *   property="description",
             *   ref="#/components/schemas/SiteDescription",
             *  ),
             *  @OA\Property (
             *   property="site_name",
             *   type="string",
             *   description="The website title",
             *  ),
             *  @OA\Property (
             *   property="subdomain",
             *   type="string",
             *   description="The subdomain name",
             *  ),
             *  @OA\Property (
             *   property="is_auto_save",
             *   type="boolean",
             *  ),
             *  @OA\Property (
             *   property="is_published",
             *   type="boolean",
             *  ),
             *  @OA\Property (
             *   property="version_id",
             *   type="integer",
             *  ),
             *  @OA\Property (
             *   property="date_time",
             *   type="string",
             *  ),
             *  @OA\Property (
             *   property="account_id",
             *   type="integer",
             *  ),
             *  @OA\Property (
             *   property="screenshot",
             *   type="string",
             *  ),
             *  @OA\Property (
             *   property="free_domain",
             *   type="string",
             *  ),
             *  @OA\Property (
             *   property="domains",
             *   type="array",
             *   @OA\Items (
             *    type="string",
             *   ),
             *  ),
             * )
             */
            return [
                self::DESCRIPTION => $description,
                "site_name" => !empty($metadata[self::SITE_NAME]) ? $metadata[self::SITE_NAME] : "",
                "site_type" => 3,
                "subdomain" => "f08b6af3c60b4037ba6a35dbbdab4283",
                "is_auto_save" => false,
                "is_published" => false,
                "version_id" => 142122,
                "date_time" => "2020-03-12T09:46:12",
                "account_id" => 28324472,
                "screenshot" => "https://storage.googleapis.com/wzresponsiveeditor-publish-latest/screenshots/45/45579192/800x500.png",
                "free_domain" => "{siteName}.websitebuilder-express.latest.wzdev.co",
                "domains" => [],
            ];
        } catch (Exception $ex) {
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

    public function is_built_with_wb4wp_($post_id)
    {
        return get_post_meta($post_id, self::WB4WP_EDIT_MODE, true);
    }

    public function publish_pages($description, $response_json, $publish_changes = true)
    {
        /**
         * Navigation mapping:
         *
         * Nav 1: navigation-1
         * Nav 2: navigation-2
         * Nav 3: navigation-3
         * Nav 4: navigation-4
         * Nav 5: navigation-5
         */
        $selected_navigation = 'navigation-1';

        /**
         * Footer mapping:
         *
         * Footer 1: bajigu80
         * Footer 2: mudoce19
         * Footer 3: bajigu77
         * Footer 4: bajigu78
         * Footer 5: bajigu79
         */
        $description_footer = $description[self::GLOBAL_SECTIONS]['footer']['layout'][self::SECTION]['id'];
        $selected_footer = !empty($description_footer) ? $description_footer : 'bajigu80';
        $ee_pages_mapping = $this->get_pages_mapping();
        $ee_pages = $response_json[self::PAGES];

        foreach ($ee_pages as &$ee_page) {
            $ee_id = $ee_page['pageId'];
            $is_main_page = $ee_page[self::MAIN_PAGE];

            if (!empty($description['pages'][0]) && !empty($description['pages'][0][self::HEADER_SECTION])) {
                $navigation_id = $description['pages'][0][self::HEADER_SECTION]['layout']['section']['id'];

                if (!empty($navigation_id)) {
                    $selected_navigation = $navigation_id;
                }
            }

            if (!array_key_exists($ee_id, $ee_pages_mapping)) {
                continue;
            }

            $id = $ee_pages_mapping[$ee_id]['id'];
            $page = get_post($id);
            // Post filtering
            remove_filter('content_save_pre', self::WP_FILTER_POST_KSES);
            remove_filter('content_filtered_save_pre', self::WP_FILTER_POST_KSES);

            $assets = [
                'baseStyle' => $ee_page['baseStyle'],
                'featureScript' => $ee_page['featureScript'],
                self::SITE_MODEL => $ee_page[self::SITE_MODEL]
            ];

            if (!empty($response_json[self::THEME][self::FONTS])) {
                $assets[self::FONTS] = $response_json[self::THEME][self::FONTS];
            }

            file_put_contents(WB4WP_PLUGIN_DIR . self::ASSETS_PATH . $id . self::JSON_EXT, wp_json_encode($assets));

            $content_sections = $ee_page[self::SECTIONS];

            $page->post_content = $this->build_post_content($content_sections, $is_main_page, $page);

            if ($is_main_page) {
                update_option('page_on_front', $page->ID);
                update_option('show_on_front', 'page');
            }

            if (!$publish_changes && $page->post_status === self::POST_PUBLISH) {
                WordPressManager::store_auto_save((array) $page);
            } else {
                wp_update_post($page);
            }

            // Post filtering
            add_filter('content_save_pre', self::WP_FILTER_POST_KSES);
            add_filter('content_filtered_save_pre', self::WP_FILTER_POST_KSES);
        }

        $current_theme_options = OptionsHelper::get(WB4WP_THEME_MODS, []);
        $current_theme_options['layout_style'] = $selected_navigation;
        $current_theme_options['layout_footer'] = $selected_footer;

        OptionsHelper::set(WB4WP_THEME_MODS, $current_theme_options);
    }

    public function handle_description_post($request)
    {
        try {
            $postDescriptionTiming = [];
            $startTime = microtime(true);
            $intervalTime = microtime(true);

            $json = $request->get_json_params();
            $site_description = $json['site_description'];
            $content = $json['published_pages'];
            $publish_changes = !empty($json['publish_changes']);

            $globalSections = $site_description[self::GLOBAL_SECTIONS];
            $theme = $site_description[self::THEME];
            $globalBinding = $site_description[self::GLOBAL_BINDING];
            $metadata = $site_description[self::METADATA];
            $pages = $site_description[self::PAGES];
            $apiKeys = $site_description['apiKeys'];
            $urls = $site_description['urls'];
            $featureStorage = $site_description['featureStorage'];
            $globalFeatureModel = $site_description['globalFeatureModel'];

            $postDescriptionTiming['read_input'] = (microtime(true) - $intervalTime);
            $intervalTime = microtime(true);

            if (!empty($json['site_name'])) {
                $this->update_site_title($json['site_name']);

                $postDescriptionTiming['set_site_title'] = (microtime(true) - $intervalTime);
                $intervalTime = microtime(true);
            }

            $nav_pages = $this->set_pages($pages, $site_description['navigation'], $publish_changes);
            $postDescriptionTiming['set_pages'] = (microtime(true) - $intervalTime) ;
            $intervalTime = microtime(true);

            $this->update_nav_menu($site_description['navigation'], $nav_pages);
            $postDescriptionTiming['update_nav_menu'] = (microtime(true) - $intervalTime);
            $intervalTime = microtime(true);

            $this->update_theme_controls($content);
            $postDescriptionTiming['update_theme_controls'] = (microtime(true) - $intervalTime);
            $intervalTime = microtime(true);

            OptionsHelper::store([
                WB4WP_GLOBAL_SECTIONS => $globalSections,
                WB4WP_THEME => $theme,
                WB4WP_GLOBAL_BINDING => $globalBinding,
                WB4WP_METADATA => $metadata,
                WB4WP_API_KEYS => $apiKeys,
                WB4WP_URLS => $urls,
                WB4WP_FEATURE_STORAGE => $featureStorage,
                WB4WP_GLOBAL_FEATURE_MODEL => $globalFeatureModel,
            ]);

            $this->publish_pages($site_description, $content, $publish_changes);
            $postDescriptionTiming['publish_pages'] = (microtime(true) - $intervalTime);
            $intervalTime = microtime(true);

            //Update the wp_options with some details from the site (like used fonts)
            $this->store_site_details($theme, $pages);
            $postDescriptionTiming['store_site_details'] = (microtime(true) - $intervalTime);
            $intervalTime = microtime(true);

            $this->store_site_meta($site_description);
            $postDescriptionTiming['store_site_meta'] = (microtime(true) - $intervalTime);
            $intervalTime = microtime(true);

            OptionsHelper::set_plugin_state('onboarded');

            StatsManager::fetch_stats(false);
            StatsManager::store_dummy_stats();

            $current_theme = wp_get_theme();
            if (empty($current_theme) || $current_theme->template != ThemeConstants::THEME_NAME) {
                ThemeManager::install_theme();
                $postDescriptionTiming['install_theme'] = (microtime(true) - $intervalTime);
            }

            $postDescriptionTiming['total_time'] = (microtime(true) - $startTime);

            // Log stats to raygun if it took longer then 15 sec
            if ((microtime(true) - $startTime) > 15) { 
                RaygunManager::get_instance()->exception_handler(new Exception(wp_json_encode($postDescriptionTiming)));
            }

            return new WP_REST_Response($postDescriptionTiming);

        } catch (Exception $ex) {
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

    /**
     * Loads the baseStyle into
     * the head of the page.
     *
     * We only do this if we are
     * on a page
     *
     * Also adds the favicon to the head
     */
    public function add_assets_to_head()
    {
        global $post;
        $favicon = OptionsHelper::get(WB4WP_SITE_FAVICON, '');

        if (!empty($favicon)) {
            echo "<link rel='shortcut icon' href='" . $favicon . "'>";
        }

        if (!empty($post)) {
            $assets = $this->get_assets($post->ID);

            if (!empty($assets)) {
                if (!empty($assets->fonts)) {
                    echo "<link rel='stylesheet' href='" . $assets->fonts . "'>";
                }

                if (!empty($assets->baseStyle)) {
                    echo "<style type='text/css'>" . $assets->baseStyle . "</style>";
                }
            }
        }

        $meta_data = OptionsHelper::get(WB4WP_METADATA, null);

        if (!empty($meta_data)) {
            if (!empty($meta_data['siteHeaderHtml'])) {
                echo $meta_data['siteHeaderHtml'];
            }

            if (!empty($meta_data[self::FAVICON_URL])) {
                echo '<link rel="icon" type="image/jpeg" href="' . $meta_data[self::FAVICON_URL] . '" />';
            }
        }
    }

    /**
     * Loads the featureScript into
     * the footer of the page.
     *
     * We only do this if we are
     * on a page
     */
    public function add_assets_to_footer()
    {
        global $post;

        if (!empty($post)) {
            $assets = $this->get_assets($post->ID);

            if (!empty($assets) && !empty($assets->featureScript)) {
                echo "<script type='text/javascript'>" . $assets->featureScript . "</script>";
            }
        }

        $meta_data = OptionsHelper::get(WB4WP_METADATA, null);

        if (!empty($meta_data)) {
            if (!empty($meta_data['siteFooterHtml'])) {
                echo $meta_data['siteFooterHtml'];
            }
        }
    }

    /**
     * Get assets data by post_id
     *
     * @param int $post_id
     *
     * @return false|mixed
     */
    public function get_assets($post_id)
    {
        $asset_file = WB4WP_PLUGIN_DIR . self::ASSETS_PATH . $post_id . self::JSON_EXT;

        if (file_exists($asset_file)) {
            return json_decode(file_get_contents($asset_file));
        }

        return false;
    }

    /**
     * Removes the asset file
     * associated with the deleted post
     *
     * @param $post_id
     *
     * @return bool
     */
    public function remove_page_assets($post_id)
    {
        $asset_file = WB4WP_PLUGIN_DIR . self::ASSETS_PATH . $post_id . self::JSON_EXT;

        if (file_exists($asset_file)) {
            return unlink($asset_file);
        }

        return true;
    }

    /**
     * Returns the pages mapping
     *
     * @return array
     */
    private function get_pages_mapping()
    {
        $pages = get_pages([self::POST_STATUS => [self::POST_DRAFT, self::POST_PUBLISH]]);
        $ee_pages_mapping = [];

        foreach ($pages as $page) {
            $id = $page->ID;

            if ($this->is_built_with_wb4wp_($id)) {
                $ee_id = get_post_meta($id, self::WB4WP_PAGE_ID, true);

                if ($ee_id === '') {
                    $ee_id = $this->id_counter++;
                    update_post_meta($id, self::WB4WP_PAGE_ID, $ee_id);
                }

                $ee_pages_mapping[$ee_id] = [
                    "ee_id" => $ee_id,
                    "id" => $id,
                    self::CAN_BE_DELETED => true,
                ];
            } else {
                $ee_pages_mapping[$id] = [
                    "id" => $id,
                    self::CAN_BE_DELETED => false,
                ];
            }
        }

        return $ee_pages_mapping;
    }

    private function get_main_page($ee_pages)
    {
        foreach ($ee_pages as $ee_page) {
            $is_main_page = !empty($ee_page[self::MAIN_PAGE]);
            if ($is_main_page) {
                return $ee_page;
            }
        }

        return null;
    }

    private function store_site_details($theme, $pages)
    {
        $appearance = [];

        //store body font
        if (!empty($theme[self::FONTS][self::BODY][self::NAME])) {
            $appearance[self::APPEARANCE_FONT_BODY] = $theme[self::FONTS][self::BODY][self::NAME];
        }

        //store heading font
        if (!empty($theme[self::FONTS][self::HEADING][self::NAME])) {
            $appearance[self::APPEARANCE_FONT_TITLE] = $theme[self::FONTS][self::HEADING][self::NAME];
        }

        //store the four theme colors
        if (!empty($theme[self::COLORS])) {
            $colors = [];
            array_push($colors, $this->colors_to_hex($theme[self::COLORS]['text']));
            array_push($colors, $this->colors_to_hex($theme[self::COLORS][self::BACKGROUND]));
            $accent = $theme[self::COLORS]['accent'];
            if (is_array($accent) && count($accent) >= 2) {
                array_push($colors, $this->colors_to_hex($accent[0]));
                array_push($colors, $this->colors_to_hex($accent[1]));
            }

            $appearance[self::APPEARANCE_COLORS] = $colors;
        }

        //find the cover image from the main page
        $main_page = $this->get_main_page($pages);
        if (!empty($main_page)) {
            if (!empty($main_page[self::HEADER_SECTION][self::BINDING][self::IMAGE][self::VALUE])) {
                $appearance[self::APPEARANCE_FEATURED_IMAGE] = $main_page[self::HEADER_SECTION][self::BINDING][self::IMAGE][self::VALUE];
            } else if (!empty($main_page[self::HEADER_SECTION][self::BINDING][self::BACKGROUND]['img'][self::VALUE])) {
                $appearance[self::APPEARANCE_FEATURED_IMAGE] = $main_page[self::HEADER_SECTION][self::BINDING][self::BACKGROUND]['img'][self::VALUE];
            }
        }

        OptionsHelper::set(WB4WP_APPEARANCE, $appearance);
    }

    private function colors_to_hex($colors)
    {
        if (count($colors) < 3) {
            return '';
        }
        return sprintf('#%02x%02x%02x', $colors[0], $colors[1], $colors[2]);
    }

    private function store_site_meta($meta)
    {
        $store_options = [];
        if (!empty($meta[self::LANGUAGE])) {
            $store_options[WB4WP_SITE_LANGUAGE] = $meta[self::LANGUAGE];
        }

        if (!empty($meta[self::GLOBAL_BINDING][self::TITLE])) {
            $store_options[WB4WP_SITE_TITLE] = $meta[self::GLOBAL_BINDING][self::TITLE];
        }

        if (!empty($meta[self::GLOBAL_BINDING][self::DESCRIPTION])) {
            $store_options[WB4WP_SITE_DESCRIPTION] = $meta[self::GLOBAL_BINDING][self::DESCRIPTION];
        }

        if (!empty($meta[self::METADATA][self::FAVICON_URL])) {
            $store_options[WB4WP_SITE_FAVICON] = $meta[self::METADATA][self::FAVICON_URL];
        }

        if (!empty($store_options)) {
            OptionsHelper::store($store_options);
        }
    }

    /**
     * Builds the post content based
     * on sections provided
     *
     * @param $sections
     * @param $is_main_page
     * @param $parent
     *
     * @return string
     */
    private function build_post_content(array $sections, $is_main_page, $parent)
    {
        $sectionIndex = 0;
        $post_content = '';

        libxml_use_internal_errors(false);

        wp_defer_term_counting(true);
        wp_defer_comment_counting(true);
        foreach ($sections as $section) {
            $sectionIndex++;
            $sectionHTML = $section['html'];

            if ($sectionIndex === 1) {
                // header
                if ($is_main_page) {
                    update_option('WB4WP_GLOBAL_HEADER', base64_encode($sectionHTML));
                }
                update_post_meta($parent->ID, 'WB4WP_PAGE_HEADER', base64_encode($sectionHTML));

            } else if ($sectionIndex === count($sections)) {
                // footer
                if ($is_main_page) {
                    update_option('WB4WP_GLOBAL_FOOTER', base64_encode($sectionHTML));
                    update_post_meta($parent->ID, 'WB4WP_PAGE_FOOTER', base64_encode($sectionHTML));
                }
            } else {
                $content = $this->generate_gutenberg_block($section);

                if (empty($content)) {
                    continue;
                }

                $post_content .= $content;
            }
        }
        wp_defer_term_counting(false);
        wp_defer_comment_counting(false);
        return $post_content;
    }

    private function generate_gutenberg_block($section)
    {
        $sectionHTML = $section['html'];
        $sectionCSS = $section['css'];

        libxml_use_internal_errors(true);
        $sectionContent = new DOMDocument();
        $sectionContent->loadHTML($sectionHTML);
        $sectionDOM = $sectionContent->getElementsByTagName(self::SECTION)[0];

        if (empty($sectionDOM)) {
            return '';
        }

        $html = '<!-- wp:html -->' . $sectionHTML . '<style scoped>' . $sectionCSS . '</style>' . '<!-- /wp:html -->';
        return '<!-- wp:wb4wp/block-generic-wb4wp-content {"id": ' . $section['id'] . '} -->' . $html . '<!-- /wp:wb4wp/block-generic-wb4wp-content -->';
    }

    /**
     * Sets the site title to $title
     *
     * @param $title
     */
    private function update_site_title($title)
    {
        global $wpdb;

        $wpdb->update("{$wpdb->prefix}options", ['option_value' => $title], ['option_name' => 'blogname']);
    }

    private function convert_to_tree( $flat, $id_field = 'ee_page_id', $parent_id_field = 'parent_id', $child_nodes_field = 'childNodes', $cur_idx = 0 )
    {
        $indexed = array();

        foreach ( $flat as $row ) {
            if ( $row->$parent_id_field === 'hidden' && $cur_idx === 0 ) {
                $indexed[$row->$id_field] = $row;
            } else if ( $row->$parent_id_field === $cur_idx ) {
                $indexed[$row->$id_field] = $row;
                $indexed[$row->$id_field]->childNodes = $this->convert_to_tree( $flat, $id_field, $parent_id_field, $child_nodes_field, $row->$id_field );
            }
        }

        return $indexed;
    }

    private function add_nav_items( $pages, $menu_id, $parent_id = 0 )
    {
        foreach ( $pages as $page ) {
            if ( $page->parent_id === 'hidden' ) {
                continue;
            }

            if ( ! empty( $page->post_title ) ) {
                $menu_object = [
                    'menu-item-title' => $page->post_title,
                    'menu-item-object-id' => $page->ID,
                    'menu-item-object' => 'page',
                    'menu-item-status' => self::POST_PUBLISH,
                    'menu-item-type' => 'post_type',
                    'menu-item-parent-id' => $parent_id,
                ];
            } else {
                $menu_object = [
                    'menu-item-title' => $page->title,
                    'menu-item-url' => $page->uriPath,
                    'menu-item-status' => self::POST_PUBLISH,
                    'menu-item-type' => 'custom',
                    'menu-item-parent-id' => $parent_id,
                ];
            }

            $new_parent_id = wp_update_nav_menu_item( $menu_id, 0, $menu_object );

            if ( $page->childNodes ) {
                $this->add_nav_items( $page->childNodes, $menu_id, $new_parent_id );
            }
        }
    }

    private function nav_menu_has_changes( $navigation_items )
    {
        $menu_items = wp_get_nav_menu_items( ProviderHelper::get_provider_name() );
        $navigation_items = array_filter( $navigation_items, function ( $item ) {
            return (!isset( $item['parentId'] ) || $item['parentId'] !== 'hidden')
                && (empty( $item['menuItem'] ) || $item['menuItem']['showInNavigation']);
        } );

        if ( !is_array( $menu_items ) || !is_array( $navigation_items ) || count( $menu_items ) !== count( $navigation_items ) ) {
            return true;
        }

        foreach ( $menu_items as $menu_item ) {
            $navigation_item_candidates = array_filter( $navigation_items, function ( $candidate ) use ( $menu_item ) {
                return !empty( $candidate['wpID'] ) && $candidate['wpID'] == $menu_item->object_id;
            } );

            if ( empty( $navigation_item_candidates ) ) {
                return true;
            }

            $navigation_item = current($navigation_item_candidates);
            if ( $this->has_navigation_item_changed( $menu_item, $navigation_item ) ) {
                return true;
            }
        }

        return false;
    }

    private function has_navigation_item_changed( $menu_item, $navigation_item )
    {
        if ( !empty( $navigation_item['menuItem'] ) && $navigation_item['menuItem']['title'] !== $menu_item->title ) {
            return true;
        }

        $no_parent_id = empty( $menu_item->parent_id ) && empty( $navigation_item['parentId'] );
        $parent_id_mismatch = empty( $menu_item->parent_id ) || empty( $navigation_item['parentId'] ) || $menu_item->parent_id != $navigation_item['parentId'];
        if ( !$no_parent_id && $parent_id_mismatch ) {
            return true;
        }

        return false;
    }

    private function update_nav_menu( $navigation_items, $nav_pages )
    {
        $navigation_items = $this->map_navigation_items( $navigation_items, $nav_pages );

        if ( $this->nav_menu_has_changes( $navigation_items ) ) {
            wp_delete_nav_menu( ProviderHelper::get_provider_name() );
            $menu_id = wp_create_nav_menu( ProviderHelper::get_provider_name() );

            $pages = [];

            foreach ( $navigation_items as $nav_item ) {
                if (
                    ! empty( $nav_item['wpID'] ) &&
                    ! empty( $nav_item[self::MENU_ITEM] ) &&
                    ! empty( $nav_item[self::MENU_ITEM][self::SHOW_IN_NAVIGATION] )

                    ||

                    ! empty( $nav_item['wpID'] ) &&
                    empty( $nav_item[self::MENU_ITEM] )
                ) {
                    $page = get_post( $nav_item['wpID'] );
                    if ( ! empty( $page) ) {
                        $page->parent_id = ! empty( $nav_item['parentId'] ) ? $nav_item['parentId'] : 0;
                        $page->ee_page_id = $nav_item['id'];
                        $pages[] = $page;
                    }
                } else if (
                    ! empty( $nav_item[self::MENU_ITEM] ) &&
                    ! empty( $nav_item[self::MENU_ITEM][self::SHOW_IN_NAVIGATION] )
                ) {
                    $external_page = ( object ) $nav_item[self::MENU_ITEM];
                    $external_page->parent_id = ! empty( $nav_item['parentId'] ) ? $nav_item['parentId'] : 0;
                    $external_page->ee_page_id = $nav_item['id'];
                    $pages[] = $external_page;
                }
            }

            $tree = $this->convert_to_tree( $pages );
            $this->add_nav_items( $tree, $menu_id );

            $locations = get_theme_mod( 'nav_menu_locations' );
            $locations['wb4wp'] = $menu_id;
            set_theme_mod( 'nav_menu_locations', $locations );
        }
    }

    /**
     * Updates the theme options
     *
     * @param $json
     */
    private function update_theme_controls( $json )
    {
        $main_page = array_values( array_filter( $json[self::PAGES], function ( $page ) {
            return ! empty( $page[self::MAIN_PAGE] );
        } ) )[0];

        if ( ! empty( $main_page ) ) {
            $site_model = $main_page[self::SITE_MODEL];
            $theme_mods = SiteModelHelper::mapSiteModelToThemeMods( $site_model );

            OptionsHelper::store( $theme_mods );
        }
    }

    private function map_navigation_items( $navigation_items, $nav_pages )
    {
        foreach ( $navigation_items as &$navigation_item ) {
            if ( empty( $navigation_item['pageId'] ) ) {
                continue;
            }

            $nav_page_index = array_search( $navigation_item['pageId'], array_column( $nav_pages, 'pageId' ) );
            if ( $nav_page_index !== false ) {
                $navigation_item['wpID'] = $nav_pages[$nav_page_index]['wpId'];
            }
        }

        // Make sure we order the array correctly
        $navigation_items = array_filter( $navigation_items, function ( $navigation_item ) {
            return isset( $navigation_item['order'] );
        } );
        $order = array_column( $navigation_items, 'order' );
        array_multisort( $order, SORT_ASC, $navigation_items );

        return $navigation_items;
    }
}