<?php

namespace Wb4Wp\Managers;

use Exception;
use Wb4Wp\Helpers\OptionsHelper;
use WP_Error;

/**
 * Class InstanceManager
 *
 * @package WB4WP\Managers
 */
final class InstanceManager
{
    /**
     * Handles the callback from the WordPress API and will store all the instance details.
     *
     * @param $request
     *
     * @return bool|WP_Error
     *
     * @throws Exception
     */
    public function handle_callback($request)
    {
        try {
            $account_information = json_decode($request->get_body());
            if ($account_information === null) {
                return new WP_Error('rest_bad_request', 'Invalid account details', ['status' => 400]);
            }

            // Store the account information in the settings
            OptionsHelper::set(WB4WP_INSTANCE_API_KEY_KEY, $account_information->api_key, true);
            OptionsHelper::store([
                WB4WP_CONNECTED_ACCOUNT_ID => $account_information->account_id,
                WB4WP_SITE_ID => $account_information->site_id,
            ]);
            OptionsHelper::set_plugin_state('provisioned');

            return true;
        } catch (Exception $ex) { 
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }
}
