<?php

namespace Wb4Wp\Managers;

use Wb4Wp\Helpers\EnvironmentHelper;
use Wb4Wp\Helpers\OptionsHelper;
use Wb4Wp\Helpers\RequestHelper;
use WP_Error;
use WP_REST_Response;

/**
 * The Bluehost manager to manage the calls for Bluehost
 *
 * Class ProviderBluehostManager
 *
 * @package WB4WP\Managers
 */
final class ProviderBluehostManager
{
    const CONTENT_TYPE = 'Content-Type';
    const APPLICATION_JSON = 'application/json';
    const HEADERS = 'headers';
    const SUCCESS = 'success';

    /**
     * Temporarily authenticate using an email address
     *
     * @param string $token
     *
     * @return WP_Error|WP_REST_Response
     */
    public static function provision($token)
    {
        if (!empty($token)) {
            $body = [
                'plugin_id' => OptionsHelper::get_instance_uuid(),
                'instance_url' => get_bloginfo('wpurl'),
                'wordpress_version' => get_bloginfo('version'),
                'plugin_version' => env('WB4WP_PLUGIN_VERSION'),
                'token' => $token,
            ];

            $response = RequestManager::post(
                EnvironmentHelper::get_app_gateway_url('wordpress/v1.0/builder'),
                [
                    self::HEADERS => [
                        self::CONTENT_TYPE => self::APPLICATION_JSON,
                    ],
                    'body' => $body,
                ]
            );

            if (!is_wp_error($response)) {
                $json = json_decode($response['body']);

                if (!empty($json->api_key)) {
                    // Store the account information in the settings
                    // TODO use the instance manager
                    OptionsHelper::set(WB4WP_INSTANCE_API_KEY_KEY, $json->api_key, true);
                    OptionsHelper::store([
                        WB4WP_CONNECTED_ACCOUNT_ID => $json->account_id,
                        WB4WP_SITE_ID => $json->site_id,
                    ]);

                    if (OptionsHelper::get(WB4WP_PLUGIN_STATE, 'new') === 'new') {
                        OptionsHelper::set_plugin_state('provisioned');
                    }

                    return new WP_REST_Response([
                        self::SUCCESS => true,
                    ]);
                }
            } else {
                // Invalid JWT Token
                return RequestHelper::throw_error(
                    [self::SUCCESS => false, 'code' => 1001],
                    __FILE__,
                    __LINE__,
                    500
                );
            }
        }

        // Invalid JWT Token
        return RequestHelper::throw_error(
            [self::SUCCESS => false, 'code' => 1000],
            __FILE__,
            __LINE__,
            500
        );
    }
}
