<?php

namespace Wb4Wp\Managers;

use WP_Error;

/**
 * Class RequestManager
 *
 * @package WB4WP\Managers
 */
final class RequestManager
{
    private static $timeout = 100,
                   $blocking = true,
                   $headers = [
                       'Content-Type' => 'application/json',
                   ];

    const HEADERS = 'headers';
    const BODY = 'body';

    /**
     * Do a post request
     *
     * @param string $url
     * @param array $args
     *
     * @return array|WP_Error
     */
    public static function post($url, array $args)
    {
        if (!empty($args[self::HEADERS])) {
            self::$headers = array_merge(self::$headers, $args[self::HEADERS]);
            unset($args[self::HEADERS]);
        }

        $default = [
            'method' => 'POST',
            'timeout' => self::$timeout,
            self::HEADERS => self::$headers,
            self::BODY => wp_json_encode([]),
            'blocking' => self::$blocking,
        ];

        if (!empty($args[self::BODY])) {
            $default[self::BODY] = wp_json_encode($args[self::BODY]);
            unset($args[self::BODY]);
        }

        return wp_remote_post($url, array_merge($default, $args));
    }

    /**
     * Do a get request
     *
     * @param string $url
     * @param array $args
     *
     * @return array|WP_Error
     */
    public static function get($url, array $args)
    {
        $default = [
            'method' => 'GET',
            'timeout' => self::$timeout,
            self::HEADERS => [],
            'blocking' => self::$blocking,
        ];

        return wp_remote_get($url, array_merge($default, $args));
    }

}