<?php

namespace Wb4Wp\Managers;

use Exception;
use WP_REST_Response;
use Wb4Wp\Helpers\EnvironmentHelper;
use Wb4Wp\Helpers\OptionsHelper;

/**
 * Class StatsManager
 *
 * @package Wb4Wp\Managers
 */
final class StatsManager
{
    const SUCCESS = 'success';

    /**
     * Fetches the stats
     *
     * @param false $cron
     *
     * @return bool|WP_REST_Response|null
     * @throws Exception
     */
    public static function fetch_stats($cron = true)
    {
        try {
            $sso_jwt = OptionsHelper::get(WB4WP_SSO_JWT_TOKEN, null, true);
            $return = null;

            if (!empty($sso_jwt)) {
                $response = RequestManager::get(
                    EnvironmentHelper::get_app_gateway_url('/wb4wp/v1.0/stats'),
                    [
                        'headers' => [
                            'Cookie' => 'cp_token=' . $sso_jwt,
                        ],
                    ]
                );

                if (!is_wp_error($response)) {
                    $json = json_decode($response['body']);
                    if (!empty($json) && (empty($json->success) || $json->success === true)) {
                        OptionsHelper::set(WB4WP_STATS, $response['body']);

                        $return = true;

                        if ($cron) {
                            $return = new WP_REST_Response(
                                [self::SUCCESS => true]
                            );
                        }

                        return $return;
                    }
                }
            }

            $return = false;

            if ($cron) {
                $return = new WP_REST_Response(
                    [self::SUCCESS => false],
                    500
                );
            }

            return $return;
        } catch (Exception $ex) { 
            RaygunManager::get_instance()->exception_handler($ex);

            // rethrow it
            throw $ex;
        }
    }

    /**
     * Stores dummy stats in the WP_Options
     */
    public static function store_dummy_stats()
    {
        OptionsHelper::set( WB4WP_STATS, json_encode( [
            "last_saved" => 1604656284926,
            "initial_page_count" => 4,
            "pages_count" => 8,
            "initial_block_count" => 20,
            "block_count" => 23,
            "created_at" => 1604656284926,
            "user_login_count" => 5,
            "route_to_wp_admin_count" => 3,
            "site_category" => "Car Dealer",
            "site_topic" => "Cars",
            "initial_template_id" => "125",
            "initial_template_name" => "business",
            "seconds_to_publish" => "562849",
        ] ) );
    }

}