<?php

use Wb4Wp\Constants\ThemeConstants;
use Wb4Wp\Helpers\ProviderHelper;
use Wb4Wp\Managers\DescriptionManager;

class DescriptionPostTest extends WP_UnitTestCase
{
    private static $class_instance,
                   $mock_request,
                   $pages_created = false,
                   $sections_created = false;

    public static function setUpBeforeClass()
    {
        parent::setUpBeforeClass();

        self::$class_instance = new DescriptionManager();

        self::$mock_request = new WP_REST_Request();
        self::$mock_request->set_header( 'Content-Type', 'application/json' );
        self::$mock_request->set_body( file_get_contents( __DIR__ . '/mock-data/description.json' ) );

        self::$class_instance->handle_description_post( self::$mock_request );
    }

    public function test_if_site_title_changed()
    {
        $this->assertSame( 'Fjkdjfs', get_option( 'blogname' ) );
    }

    public function test_if_pages_were_created()
    {
        $posts = get_posts( [
            'post_type' => 'page',
            'post_status' => 'draft',
        ] );

        $this->assertNotEmpty( $posts );

        if ( ! empty( $posts ) ) {
            self::$pages_created = true;
        }
    }

    public function test_if_created_pages_are_not_empty()
    {
        if ( ! self::$pages_created ) {
            $this->markTestSkipped( 'No pages created, skipping content check.' );
            return;
        }

        $posts = get_posts( [
            'post_type' => 'page',
            'post_status' => 'draft',
        ] );

        foreach ( $posts as $post ) {
            $this->assertNotEmpty( $post->post_content );
        }
    }

    public function test_if_wp_navigation_menu_was_created()
    {
        $menu = wp_get_nav_menu_object( ProviderHelper::get_provider_name() );

        $this->assertNotFalse( $menu );
    }

    public function test_if_theme_mods_were_stored()
    {
        $this->assertNotNull( get_option( ThemeConstants::THEME_MODS_NAME ) );
    }

    public function test_if_global_sections_were_stored()
    {
        $this->assertNotNull( get_option( WB4WP_GLOBAL_SECTIONS ) );
    }

    public function test_if_theme_meta_was_stored()
    {
        $this->assertNotNull( get_option( WB4WP_THEME ) );
    }

    public function test_if_meta_data_was_stored()
    {
        $this->assertNotNull( get_option( WB4WP_METADATA ) );
    }

    public function test_if_api_keys_were_stored()
    {
        $this->assertNotNull( get_option( WB4WP_API_KEYS ) );
    }

    public function test_if_urls_were_stored()
    {
        $this->assertNotNull( get_option( WB4WP_URLS ) );
    }

    public function test_if_feature_storage_was_stored()
    {
        $this->assertNotNull( get_option( WB4WP_FEATURE_STORAGE ) );
    }

    public function test_if_global_feature_model_was_stored()
    {
        $this->assertNotNull( get_option( WB4WP_GLOBAL_FEATURE_MODEL ) );
    }

    public function test_if_site_details_were_stored()
    {
        $this->assertNotEmpty( get_option( WB4WP_APPEARANCE ) );
    }

    public function test_if_site_language_is_stored()
    {
        $this->assertNotEmpty( get_option( WB4WP_SITE_LANGUAGE ) );
    }

    public function test_if_site_title_is_stored()
    {
        $this->assertNotEmpty( get_option( WB4WP_SITE_TITLE ) );
    }

    public function test_if_plugin_state_is_onboarded()
    {
        $this->assertSame( 'onboarded', get_option( WB4WP_PLUGIN_STATE ) );
    }

    public function test_if_stats_were_retrieved_and_stored()
    {
        $this->assertNotNull( get_option( WB4WP_STATS ) );
    }

}