// webpack.config.js
const defaultConfig = require("./node_modules/@wordpress/scripts/config/webpack.config");
require('dotenv').config()
const path = require('path')
const MiniCssExtractPlugin = require('mini-css-extract-plugin');

module.exports = {
    ...defaultConfig,
    plugins: [
        new MiniCssExtractPlugin(),
        ...defaultConfig.plugins,
    ],
    module: {
        ...defaultConfig.module,
        rules: [
            ...defaultConfig.module.rules,
            {
                test: /\.(png|jpe?g|gif)$/i,
                use: [
                    {
                        loader: 'url-loader',
                    },
                ],
            },
            {
                test: /\.ttf(\?v=\d+\.\d+\.\d+)?$/,
                exclude: path.resolve(__dirname, 'node_modules'),
                use: [
                    {
                        loader: 'url-loader',
                        options: {
                            prefix: 'font',
                            limit: 10000,
                            mimetype: 'application/octet-stream'
                        }
                    }
                ]
            },
            {
                test: /\.s[ac]ss$/i,
                use: [
                    MiniCssExtractPlugin.loader,
                    'css-loader',
                    'postcss-loader',
                    {
                        loader: 'sass-loader',
                        options: {
                            additionalData: `
                        $WB4WP_APP_URL: "${process.env.WB4WP_APP_URL}";
                      `,
                        },
                    },
                ],
            }
        ]
    },
    output: {
        ...defaultConfig.output,
        filename: '[name].js'
    },
    entry: {
        'new-page-empty-state-page-type-selector': './src/blocks/new-page-empty-state-page-type-selector',
        'block-generic-wb4wp-content': './src/blocks/block-generic-wb4wp-content',
        'open-wb4wp-editor': './src/blocks/open-wb4wp-editor',
        onboarding: './src/blocks/onboarding'
    },
    externals: {
        "react": "React",
        "react-dom": "ReactDOM"
    }
};