<?php
/**
 * Website Builder Theme functions and definitions
 *
 * @link https://developer.wordpress.org/themes/basics/theme-functions/
 *
 * @package Website Builder Theme
 */

use Wb4WpTheme\Managers\CustomizeManager;
use Wb4WpTheme\Managers\TemplateManager;

if ( !defined( '_WEBSITE_BUILDER_THEME_VERSION' ) ) {
    // Replace the version number of the theme on each release.
    define( '_WEBSITE_BUILDER_THEME_VERSION', '1.0.0' );
}

// Load all the required files
if ( file_exists( __DIR__ . '/vendor/autoload.php' ) ) {
    require_once __DIR__ . '/vendor/autoload.php';
}

define( 'WB4WP_PROVIDER_NAME', 'Website Builder' );

function wb4wp_theme_enqueue_style()
{   
    $theme_name = 'wb4wp-theme';

    wp_enqueue_style($theme_name, get_template_directory_uri() . '/dist/main.css', false );

    (new TemplateManager())->render_template_assets($theme_name);
}

add_action( 'wp_enqueue_scripts', 'wb4wp_theme_enqueue_style' );

if ( !function_exists( 'wb4wp_theme_setup' ) ) :

    function wb4wp_theme_setup()
    {
        /*
         * Let WordPress manage the document title.
         * By adding theme support, we declare that this theme does not use a
         * hard-coded <title> tag in the document head, and expect WordPress to
         * provide it for us.
         */
        add_theme_support( 'title-tag' );

        /*
         * Enable support for Post Thumbnails on posts and pages.
         *
         * @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
         */
        add_theme_support( 'post-thumbnails' );

        // This theme uses wp_nav_menu() in one location.
        register_nav_menus( [
            'wb4wp' => esc_html__( 'Primary', 'wb4wp_theme' ),
        ] );

        /*
         * Switch default core markup for search form, comment form, and comments
         * to output valid HTML5.
         */
        add_theme_support(
            'html5',
            array(
                'search-form',
                'comment-form',
                'comment-list',
                'gallery',
                'caption',
                'style',
                'script',
            )
        );
    }
endif;

add_action( 'after_setup_theme', 'wb4wp_theme_setup' );

function wb4wp_theme_customizer($wp_customize)
{
    $customize_manager = new CustomizeManager( $wp_customize );
    $customize_manager->add_customizer_controls();
}
add_action( 'customize_register', 'wb4wp_theme_customizer' );

function wb4wp_customize_checkbox_fix() {
    wp_enqueue_script(
        'wb4wp_customize_checkbox_fix',
        get_template_directory_uri() . '/src/customizer/customize-checkbox-fix.js',
        ['jquery', 'customize-controls'],
        false,
        true
    );
}
add_action( 'customize_controls_enqueue_scripts', 'wb4wp_customize_checkbox_fix' );

function wb4wp_add_menu_pages()
{
    global $submenu;

    $submenu['themes.php'][] = [
        "Edit " . WB4WP_PROVIDER_NAME . " settings",
        'edit_theme_options',
        admin_url( 'admin.php?page=wb4wp-editor&wp-edit-link=/appearance' )
    ];
}

add_action( 'admin_menu', 'wb4wp_add_menu_pages' );

// This theme support page wide sections
function support_wide_mode()
{
    add_theme_support( 'align-wide' );
}

add_action( 'init', 'support_wide_mode' );

function wb4wp_theme_customizer_js()
{

    wp_enqueue_script(
        'wb4wp-theme-customizer-js',
        get_template_directory_uri() . '/src/customizer/customizer.js',
        ['jquery'],
        '1.0.0',
        true
    );

    wp_localize_script(
        'wb4wp-theme-customizer-js',
        'wb4wpL10n',
        ['title_text' => sprintf( __( 'Open %s', 'wb4wp_theme' ), WB4WP_PROVIDER_NAME )]
    );

}

add_action( 'customize_controls_enqueue_scripts', 'wb4wp_theme_customizer_js' );
