<?php

namespace Wb4WpTheme\Managers;


use WP_Customize_Color_Control;

abstract class CustomizeControls
{
    protected $wp_customize;

    const SOCIAL_BUTTONS = 'social_buttons';
    const SITE_TITLE = 'site_title';
    const DESCRIPTION = 'description';
    const FIXED_NAVIGATION_BAR = 'fixed_navigation_bar';
    const PHONE_NUMBER = 'phone_number';
    const EMAIL = 'email';
    const STREET = 'street';
    const CITY = 'city';
    const ZIP_CODE = 'zip_code';
    const STATE = 'state';
    const COUNTRY = 'country';
    const COPYRIGHT_MESSAGE = 'copyright_message';
    const LINK_TO_SITEMAP = 'link_to_sitemap';
    const PAGE_MENU = 'page_menu';

    public function __construct( $wp_customize )
    {
        $this->wp_customize = $wp_customize;
    }

    public static function get_setting_name( $section_name, $setting_name )
    {
        return $section_name . '_' . $setting_name . '_setting';
    }

    /**
     * Adds a setting and a control to the Wp Theme costumizer api.
     *
     * @param $section_name string
     * @param $setting_name string
     * @param $control_settings array
     */
    protected function add_setting_with_control( $section_name, $setting_name, $control_settings )
    {
        $setting_name = $this->get_setting_name( $section_name, $setting_name );
        $default_value = CustomizeManager::DEFAULT_SETTING_VALUES[$setting_name];

        switch ( $control_settings['type'] ) {
            case 'color':
                $this->wp_customize->add_setting( $setting_name, [
                    'type' => 'option',
                    'default' => $default_value,
                    'transport' => 'refresh',
                    'sanitize_callback' => 'sanitize_hex_color',
                ] );

                $this->wp_customize->add_control(
                    new WP_Customize_Color_Control(
                        $this->wp_customize,
                        $setting_name, array_merge( $control_settings, [
                            'section' => $section_name,
                            'settings' => $setting_name
                        ] )
                    )
                );
                break;

            case 'checkbox':
                $this->wp_customize->add_setting( $setting_name, [
                    'type' => 'option',
                    'default' => $default_value,
                    'transport' => 'refresh',
                    'sanitize_callback' => function ( $checked ) {
                        return $checked ? 'true' : 'false';
                    }
                ] );

                $this->wp_customize->add_control( $setting_name . '_control', array_merge( $control_settings, [
                    'section' => $section_name,
                    'settings' => $setting_name
                ] ) );
                break;

            default:
                $this->wp_customize->add_setting( $setting_name, [
                    'type' => 'option',
                    'default' => $default_value,
                    'transport' => 'refresh'
                ] );

                $this->wp_customize->add_control( $setting_name . '_control', array_merge( $control_settings, [
                    'section' => $section_name,
                    'settings' => $setting_name
                ] ) );
        }
    }

    protected function add_social_buttons_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::SOCIAL_BUTTONS, [
            'type' => 'checkbox',
            'label' => __( 'Social buttons', 'wb4wp_theme' ),
            'description' => __( 'Display social media buttons', 'wb4wp_theme' ),
        ] );
    }

    protected function add_site_title_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::SITE_TITLE, [
            'type' => 'checkbox',
            'label' => __( 'Site title', 'wb4wp_theme' ),
            'description' => __( 'Display the site title', 'wb4wp_theme' ),
        ] );
    }

    protected function add_description_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::DESCRIPTION, [
            'type' => 'text',
            'label' => __( 'Site description', 'wb4wp_theme' ),
            'description' => __( 'add a site description here', 'wb4wp_theme' ),
        ] );
    }

    protected function add_fixed_navigation_bar_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::FIXED_NAVIGATION_BAR, [
            'type' => 'checkbox',
            'label' => __( 'Fixed navigation bar', 'wb4wp_theme' ),
            'description' => __( 'Keep navigation bar fixed to the top of the screen when scrolling', 'wb4wp_theme' ),
        ] );
    }

    protected function add_phone_number_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::PHONE_NUMBER, [
            'type' => 'text',
            'label' => __( 'Phone Number', 'wb4wp_theme' ),
            'description' => __( 'add a phone number here', 'wb4wp_theme' ),
        ] );
    }

    protected function add_email_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::EMAIL, [
            'type' => 'text',
            'label' => __( 'E-mail', 'wb4wp_theme' ),
            'description' => __( 'add a email here', 'wb4wp_theme' ),
        ] );
    }

    protected function add_street_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::STREET, [
            'type' => 'text',
            'label' => __( 'Street', 'wb4wp_theme' ),
            'description' => __( 'add a street here', 'wb4wp_theme' ),
        ] );
    }

    protected function add_city_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::CITY, [
            'type' => 'text',
            'label' => __( 'City', 'wb4wp_theme' ),
            'description' => __( 'add a city here', 'wb4wp_theme' ),
        ] );
    }

    protected function add_zip_code_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::ZIP_CODE, [
            'type' => 'text',
            'label' => __( 'Zipcode', 'wb4wp_theme' ),
            'description' => __( 'add a zip_code here', 'wb4wp_theme' ),
        ] );
    }

    protected function add_state_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::STATE, [
            'type' => 'text',
            'label' => __( 'State', 'wb4wp_theme' ),
            'description' => __( 'add a state here', 'wb4wp_theme' ),
        ] );
    }

    protected function add_country_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::COUNTRY, [
            'type' => 'text',
            'label' => __( 'Country', 'wb4wp_theme' ),
            'description' => __( 'add a country here', 'wb4wp_theme' ),
        ] );
    }

    protected function add_copyright_message_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::COPYRIGHT_MESSAGE, [
            'type' => 'checkbox',
            'label' => __( 'Copyright message', 'wb4wp_theme' ),
            'description' => __( 'Display the copyright message', 'wb4wp_theme' ),
        ] );
    }

    protected function add_link_to_sitemap_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::LINK_TO_SITEMAP, [
            'type' => 'checkbox',
            'label' => __( 'Link to Sitemap', 'wb4wp_theme' ),
            'description' => __( 'Display the link to the sitemap', 'wb4wp_theme' ),
        ] );
    }

    protected function add_page_menu_control( $section_name )
    {
        $this->add_setting_with_control( $section_name, self::PAGE_MENU, [
            'type' => 'checkbox',
            'label' => __( 'Page Menu', 'wb4wp_theme' ),
            'description' => __( 'Display the page menu', 'wb4wp_theme' ),
        ] );
    }
}