<?php

namespace Wb4WpTheme\Managers;

use InvalidArgumentException;

final class CustomizeManager extends CustomizeControls
{
    const DEFAULT_SETTING_VALUES = [
        'wb4wp_header_section_layout_setting' => 'navigation-1',
        'wb4wp_header_section_social_buttons_setting' => 'true',
        'wb4wp_header_section_site_title_setting' => 'true',
        'wb4wp_header_section_fixed_navigation_bar_setting' => 'true',
        'wb4wp_footer_section_layout_setting' => 'footer-1',
        'wb4wp_footer_section_social_buttons_setting' => 'true',
        'wb4wp_footer_section_site_title_setting' => 'true',
        'wb4wp_footer_section_description_setting' => 'Add a description here',
        'wb4wp_footer_section_copyright_message_setting' => 'false',
        'wb4wp_footer_section_link_to_sitemap_setting' => 'true',
        'wb4wp_footer_section_page_menu_setting' => 'true',
        'wb4wp_social_accounts_section_facebook_setting' => '',
        'wb4wp_social_accounts_section_twitter_setting' => '',
        'wb4wp_social_accounts_section_instagram_setting' => '',
        'wb4wp_social_accounts_section_linkedin_setting' => '',
        'wb4wp_social_accounts_section_pinterest_setting' => '',
        'wb4wp_social_accounts_section_youtube_setting' => '',
        'wb4wp_social_accounts_section_opentable_setting' => '',
        'wb4wp_contact_information_section_phone_number_setting' => '0123456789',
        'wb4wp_contact_information_section_email_setting' => 'info@example.com',
        'wb4wp_contact_information_section_street_setting' => '10 Corporate Drive',
        'wb4wp_contact_information_section_city_setting' => 'Burlington',
        'wb4wp_contact_information_section_zip_code_setting' => '01803',
        'wb4wp_contact_information_section_state_setting' => 'MA',
        'wb4wp_contact_information_section_country_setting' => 'United States',
        'wb4wp_color_section_accent1_setting' => '#4f8a8b',
        'wb4wp_color_section_accent2_setting' => '#fbd46d',
        'wb4wp_color_section_background_setting' => '#eeeeee',
        'wb4wp_color_section_text_setting' => '#222831',
        'wb4wp_fonts_section_body_setting' => 'Rubik:400',
        'wb4wp_fonts_section_heading_setting' => 'Old Standard TT:400',
        'wb4wp_fonts_section_font_size_setting' => '1',
        'wb4wp_logo_section_url_setting' => '',
        'wb4wp_logo_section_show_in_header_setting' => 'false',
        'wb4wp_logo_section_show_in_footer_setting' => 'false',
    ];

    /**
     * Retrieves a single setting, if set, returns default value otherwise.
     *
     * @param $name string
     *
     * @return mixed
     *
     * @throws InvalidArgumentException
     */
    public static function get_setting( $name )
    {
        $default_setting_values = self::DEFAULT_SETTING_VALUES;
        $setting_default = isset( $default_setting_values[$name] ) ? $default_setting_values[$name] : null;
        $setting = get_option( $name, $setting_default );

        switch ( $setting ) {
            case 'true':
                return true;
            case 'false':
                return false;
            default:
                return $setting;
        }
    }

    public static function before ($str, $inthat)
    {
        return substr($inthat, 0, strpos($inthat, $str));
    }

    public static function after ($str, $inthat)
    {
        if (!is_bool(strpos($inthat, $str))){
            return substr($inthat, strpos($inthat,$str)+strlen($str));
        }
        return $str;
    }

    public function add_customizer_controls()
    {
        $this->add_header_controls();
        $this->add_footer_controls();
        $this->add_social_accounts_controls();
        $this->add_contact_information_controls();
        $this->add_color_controls();
        $this->add_font_controls();
        $this->add_logo_controls();
    }

    private function add_header_controls()
    {
        $section_name = 'wb4wp_header_section';

        $this->wp_customize->add_section( $section_name, [
            'priority' => 2,
            'title' => __( 'Header', 'wb4wp_theme' ),
            'description' => __( 'Customize your header', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'layout', [
            'type' => 'select',
            'label' => __( 'Layout Style', 'wb4wp_theme' ),
            'description' => __( 'The layout style to use', 'wb4wp_theme' ),
            'choices' => [
                'navigation-1' => 'Navigation 1',
                'navigation-2' => 'Navigation 2',
                'navigation-3' => 'Navigation 3',
                'navigation-4' => 'Navigation 4',
                'navigation-5' => 'Navigation 5'
                // 'navigation-6' => 'Navigation 6',  disable this menu item
            ],
        ] );

        $this->add_social_buttons_control( $section_name );
        $this->add_site_title_control( $section_name );
        $this->add_fixed_navigation_bar_control( $section_name );
    }

    private function add_font_controls()
    {
        $section_name = 'wb4wp_fonts_section';

        $this->wp_customize->add_section( $section_name, [
            'priority' => 5,
            'title' => __( 'Fonts', 'wb4wp_theme' ),
            'description' => __( 'Customize your fonts', 'wb4wp_theme' ),
        ] );

        $font_choices = [
            'Abril Fatface:400' => 'Abril Fatface:400',
            'Alata:400' => 'Alata:400',
            'Alegreya Sans:400' => 'Alegreya Sans:400',
            'Alfa Slab One:400' => 'Alfa Slab One:400',
            'Amatic SC:400' => 'Amatic SC:400',
            'Anton:400' => 'Anton:400',
            'Arvo:400' => 'Arvo:400',
            'Assistant:600' => 'Assistant:600',
            'Atma:700' => 'Atma:700',
            'Audiowide:400' => 'Audiowide:400',
            'Barlow:500' => 'Barlow:500',
            'Bevan:400' => 'Bevan:400',
            'Bree Serif:400' => 'Bree Serif:400',
            'Calistoga:400' => 'Calistoga:400',
            'Cardo:400' => 'Cardo:400',
            'Catamaran:600' => 'Catamaran:600',
            'Caveat:400' => 'Caveat:400',
            'Chewy:400' => 'Chewy:400',
            'Chivo:700' => 'Chivo:700',
            'Cinzel:700' => 'Cinzel:700',
            'Coiny:400' => 'Coiny:400',
            'Cormorant Garamond:700' => 'Cormorant Garamond:700',
            'Dancing Script:400' => 'Dancing Script:400',
            'Domine:400' => 'Domine:400',
            'Dosis:400' => 'Dosis:400',
            'Eczar:600' => 'Eczar:600',
            'Fira Sans:600' => 'Fira Sans:600',
            'Frank Ruhl Libre:400' => 'Frank Ruhl Libre:400',
            'Fredericka the Great:400' => 'Fredericka the Great:400',
            'Fredoka One:400' => 'Fredoka One:400',
            'Goudy Book Letter:400' => 'Goudy Book Letter:400',
            'Heebo:500' => 'Heebo:500',
            'Heebo:800' => 'Heebo:800',
            'Indie Flower:400' => 'Indie Flower:400',
            'Josefin sans:400' => 'Josefin sans:400',
            'Julius Sans One:400' => 'Julius Sans One:400',
            'Lato:400' => 'Lato:400',
            'Lato:lic' => 'Lato:lic',
            'Libre Baskerville:700' => 'Libre Baskerville:700',
            'Lobster Two:400' => 'Lobster Two:400',
            'Luckiest Guy:400' => 'Luckiest Guy:400',
            'Merriweather:400' => 'Merriweather:400',
            'Merriweather Sans:400' => 'Merriweather Sans:400',
            'Montserrat:400' => 'Montserrat:400',
            'Montserrat:800' => 'Montserrat:800',
            'Muli:400' => 'Muli:400',
            'Mulish:300' => 'Mulish:300',
            'Neuton:400' => 'Neuton:400',
            'Nunito:300' => 'Nunito:300',
            'Old Standard TT:400' => 'Old Standard TT:400',
            'Oswald:300' => 'Oswald:300',
            'Oswald:400' => 'Oswald:400',
            'Ovo:400' => 'Ovo:400',
            'Pacifico:400' => 'Pacifico:400',
            'Parisienne:400' => 'Parisienne:400',
            'Patua One:400' => 'Patua One:400',
            'Permanent Marker:400' => 'Permanent Marker:400',
            'Petit Formal Script:400' => 'Petit Formal Script:400',
            'Playfair Display:400' => 'Playfair Display:400',
            'Playfair Display:700' => 'Playfair Display:700',
            'Playfair Display Black:800' => 'Playfair Display Black:800',
            'Prata:400' => 'Prata:400',
            'Press Start 2P:400' => 'Press Start 2P:400',
            'Princess Sofia:400' => 'Princess Sofia:400',
            'PT Sans:400' => 'PT Sans:400',
            'PT Sans:700' => 'PT Sans:700',
            'PT Sans Caption:700' => 'PT Sans Caption:700',
            'Questrial:400' => 'Questrial:400',
            'Raleway:300' => 'Raleway:300',
            'Red Hat Display:700' => 'Red Hat Display:700',
            'Red Hat Display:900' => 'Red Hat Display:900',
            'Roboto:600' => 'Roboto:600',
            'Roboto:700' => 'Roboto:700',
            'Roboto Condensed:700' => 'Roboto Condensed:700',
            'Roboto Slab:400' => 'Roboto Slab:400',
            'Rosario:400' => 'Rosario:400',
            'Rubik:400' => 'Rubik:400',
            'Ruda:400' => 'Ruda:400',
            'Rufina:700' => 'Rufina:700',
            'Sacramento:400' => 'Sacramento:400',
            'Satisfy:400' => 'Satisfy:400',
            'Shrikhand:400' => 'Shrikhand:400',
            'Slabo:400' => 'Slabo:400',
            'Solway:400' => 'Solway:400',
            'Source Sans Pro:400' => 'Source Sans Pro:400',
            'Source Sans Pro:800' => 'Source Sans Pro:800',
            'Source Serif Pro:400' => 'Source Serif Pro:400',
            'Special Elite:400' => 'Special Elite:400',
            'Ubuntu:700' => 'Ubuntu:700',
            'Unica One:400' => 'Unica One:400',
            'Varela Round:400' => 'Varela Round:400',
            'Volkhov:400' => 'Volkhov:400',
            'Vollkorn:400' => 'Vollkorn:400',
            'Work Sans:300' => 'Work Sans:300',
        ];

        $this->add_setting_with_control( $section_name, 'heading', [
            'type' => 'select',
            'label' => __( 'Heading font', 'wb4wp_theme' ),
            'description' => __( 'The heading font used in the header and footer', 'wb4wp_theme' ),
            'choices' => $font_choices,
        ] );

        $this->add_setting_with_control( $section_name, 'body', [
            'type' => 'select',
            'label' => __( 'Body font', 'wb4wp_theme' ),
            'description' => __( 'The body google font used in the header and footer', 'wb4wp_theme' ),
            'choices' => $font_choices,
        ] );

        $this->add_setting_with_control( $section_name, 'font_size', [
            'type' => 'select',
            'label' => __( 'Font size', 'wb4wp_theme' ),
            'description' => __( 'Set the font size for your website', 'wb4wp_theme' ),
            'choices' => [
                '0.875' => 'Small',
                '1' => 'Medium',
                '1.125' => 'Large'
            ],
        ] );
    }

    private function add_logo_controls()
    {
        $section_name = 'wb4wp_logo_section';

        $this->wp_customize->add_section( $section_name, [
            'priority' => 4,
            'title' => __( 'Logo', 'wb4wp_theme' ),
            'description' => __( 'Customize your logo', 'wb4wp_theme' ),
        ]);

        $this->add_setting_with_control( $section_name, 'url', [
            'type' => 'text',
            'label' => __( 'Logo URL', 'wb4wp_theme' ),
            'description' => __( 'Enter the URL to your logo here', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'show_in_header', [
            'type' => 'checkbox',
            'label' => __( 'Show in the header', 'wb4wp_theme' ),
            'description' => __( 'Has no effect when the logo URL has not been configured', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'show_in_footer', [
            'type' => 'checkbox',
            'label' => __( 'Show in the footer', 'wb4wp_theme' ),
            'description' => __( 'Has no effect when the logo URL has not been configured', 'wb4wp_theme' ),
        ] );
    }

    private function add_color_controls()
    {
        $section_name = 'wb4wp_color_section';

        $this->wp_customize->add_section( $section_name, [
            'priority' => 4,
            'title' => __( 'Colors', 'wb4wp_theme' ),
            'description' => __( 'Customize your colors', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'accent1', [
            'type' => 'color',
            'label' => __( 'Accent 1', 'wb4wp_theme' ),
            'description' => __( 'The primary color', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'accent2', [
            'type' => 'color',
            'label' => __( 'Accent 2', 'wb4wp_theme' ),
            'description' => __( 'An accent color', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'background', [
            'type' => 'color',
            'label' => __( 'Background', 'wb4wp_theme' ),
            'description' => __( 'An background color', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'text', [
            'type' => 'color',
            'label' => __( 'Text', 'wb4wp_theme' ),
            'description' => __( 'The text color', 'wb4wp_theme' ),
        ] );
    }

    private function add_footer_controls()
    {
        $section_name = 'wb4wp_footer_section';

        $this->wp_customize->add_section( $section_name, [
            'priority' => 3,
            'title' => __( 'Footer', 'wb4wp_theme' ),
            'description' => __( 'Customize your footer', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'layout', [
            'type' => 'select',
            'label' => __( 'Layout Footer', 'wb4wp_theme' ),
            'description' => __( 'The layout footer to use', 'wb4wp_theme' ),
            'choices' => [
                'footer-1' => 'Footer 1',
                'footer-2' => 'Footer 2',
                'footer-3' => 'Footer 3',
                'footer-4' => 'Footer 4',
                'footer-5' => 'Footer 5',
            ],
        ] );

        $this->add_social_buttons_control($section_name);
        $this->add_site_title_control($section_name);
        $this->add_description_control($section_name);
        $this->add_copyright_message_control($section_name);
        $this->add_link_to_sitemap_control($section_name);
        $this->add_page_menu_control($section_name);
    }

    private function add_contact_information_controls()
    {
        $section_name = 'wb4wp_contact_information_section';

        $this->wp_customize->add_section( $section_name, [
            'priority' => 3,
            'title' => __( 'Contact Information', 'wb4wp_theme' ),
            'description' => __( 'Customize your contact information', 'wb4wp_theme' ),
        ] );

        $this->add_phone_number_control($section_name);
        $this->add_email_control($section_name);
        $this->add_street_control($section_name);
        $this->add_city_control($section_name);
        $this->add_zip_code_control($section_name);
        $this->add_state_control($section_name);
        $this->add_country_control($section_name);
    }

    private function add_social_accounts_controls()
    {
        $section_name = 'wb4wp_social_accounts_section';

        $this->wp_customize->add_section( $section_name, [
            'priority' => 3,
            'title' => __( 'Social Accounts', 'wb4wp_theme' ),
            'description' => __( "Add the URLs to your social accounts here. These will be used when using the 'Social buttons' option in the header and/or footer, allowing users to go to your social accounts directly.", 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'facebook', [
            'type' => 'text',
            'label' => __( 'Facebook', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'twitter', [
            'type' => 'text',
            'label' => __( 'Twitter', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'instagram', [
            'type' => 'text',
            'label' => __( 'Instagram', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'linkedin', [
            'type' => 'text',
            'label' => __( 'LinkedIn', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'pinterest', [
            'type' => 'text',
            'label' => __( 'Pinterest', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'youtube', [
            'type' => 'text',
            'label' => __( 'YouTube', 'wb4wp_theme' ),
        ] );

        $this->add_setting_with_control( $section_name, 'opentable', [
            'type' => 'text',
            'label' => __( 'OpenTable', 'wb4wp_theme' ),
        ] );
    }
}